{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agControlBar;

interface

uses Messages, Windows, Classes, Controls, Graphics, Forms, ExtCtrls;

type
  TagBandPaintOption = (bpoGrabber, bpoFrame);
  TagBandPaintOptions = set of TagBandPaintOption;

  TagBandDragEvent = procedure (Sender: TObject; Control: TControl;
    var Drag: Boolean) of object;
  TagBandInfoEvent = procedure (Sender: TObject; Control: TControl;
    var Insets: TRect; var PreferredSize, RowCount: Integer) of object;
  TagBandMoveEvent = procedure (Sender: TObject; Control: TControl;
    var ARect: TRect) of object;
  TagBandPaintEvent = procedure (Sender: TObject; Control: TControl;
    Canvas: TCanvas; var ARect: TRect; var Options: TagBandPaintOptions) of object;

  TagRowSize = 1..MaxInt;

  {$IFDEF D4}
  TagCustomControlBar = class(TCustomControl)
  {$ELSE}
  TagCustomControlBar = class(TCustomPanel)
  {$ENDIF}
  private
    FAligning: Boolean;
    FAutoDrag: Boolean;
    FDockingControl: TControl;
    FDragControl: TControl;
    FDragOffset: TPoint;
    FDrawing: Boolean;
    FItems: TList;
    FPicture: TPicture;
    FRowSize: TagRowSize;
    FRowSnap: Boolean;
    FVertical: Boolean;
    FOnBandDrag: TagBandDragEvent;
    FOnBandInfo: TagBandInfoEvent;
    FOnBandMove: TagBandMoveEvent;
    FOnBandPaint: TagBandPaintEvent;
    FOnPaint: TNotifyEvent;
    {$IFDEF D4}
    FFloating: Boolean;
    {$ELSE}
    FAutoSize: boolean;
    FDrawBevel: boolean;
    procedure SetAutoSize(Value: boolean);
    {$ENDIF}
    procedure ResetDockItems;
    procedure DoAlignControl(AControl: TControl);
    function FindPos(AControl: TControl): Pointer;
    function HitTest2(X, Y: Integer): Pointer;
    procedure DockControl(AControl: TControl; const ARect: TRect;
      BreakList, IndexList, SizeList: TList; Parent: Pointer;
      ChangedPriorBreak: Boolean; Insets: TRect; PreferredSize,
      RowCount: Integer; Existing: Boolean);
    procedure PictureChanged(Sender: TObject);
    procedure SetPicture(const Value: TPicture);
    procedure SetRowSize(Value: TagRowSize);
    procedure SetRowSnap(Value: Boolean);
    procedure SetVertical(Value: Boolean);
    procedure UnDockControl(AControl: TControl);
    function UpdateItems(AControl: TControl): Boolean;
    procedure CMControlListChange(var Message: TCMControlListChange); message CM_CONTROLLISTCHANGE;
    procedure CMDesignHitTest(var Message: TCMDesignHitTest); message CM_DESIGNHITTEST;
    procedure CNKeyDown(var Message: TWMKeyDown); message CN_KEYDOWN;
  protected
    procedure AlignControls(AControl: TControl; var ARect: TRect); override;
    function CanAutoSize(var NewWidth, NewHeight: Integer): Boolean; {$IFDEF D4}override{$ELSE}virtual{$ENDIF};
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DoBandMove(Control: TControl; var ARect: TRect); virtual;
    procedure DoBandPaint(Control: TControl; Canvas: TCanvas; var ARect: TRect;
      var Options: TagBandPaintOptions); virtual;
    function DoPaletteChange: Boolean;
    function DragControl(AControl: TControl; X, Y: Integer;
      KeepCapture: Boolean{$IFDEF D4} = False{$ENDIF}): Boolean; virtual;
    procedure GetControlInfo(AControl: TControl; var Insets: TRect;
      var PreferredSize, RowCount: Integer); virtual;
    function GetPalette: HPALETTE; override;
    function HitTest(X, Y: Integer): TControl;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Paint; override;
    procedure PaintControlFrame(Canvas: TCanvas; AControl: TControl;
      var ARect: TRect); virtual;
    property AutoDrag: Boolean read FAutoDrag write FAutoDrag default True;
    property RowSize: TagRowSize read FRowSize write SetRowSize default 26;
    property RowSnap: Boolean read FRowSnap write SetRowSnap default True;
    property Vertical: Boolean read FVertical write SetVertical default False;
    property OnBandDrag: TagBandDragEvent read FOnBandDrag write FOnBandDrag;
    property OnBandInfo: TagBandInfoEvent read FOnBandInfo write FOnBandInfo;
    property OnBandMove: TagBandMoveEvent read FOnBandMove write FOnBandMove;
    property OnBandPaint: TagBandPaintEvent read FOnBandPaint write FOnBandPaint;
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
    property AutoSize{$IFNDEF D4}: boolean read FAutoSize write SetAutoSize{$ENDIF};
    {$IFDEF D4}
    property DockSite default True;
    procedure DockOver(Source: TDragDockObject; X, Y: Integer; State: TDragState;
      var Accept: Boolean); override;
    procedure GetSiteInfo(Client: TControl; var InfluenceRect: TRect;
      MousePos: TPoint; var CanDock: Boolean); override;
    property BevelKind default bkTile;
    {$ELSE}
    procedure AdjustSize; virtual;
    function GetClientRect: TRect; override;
    {$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure StickControls; virtual;
    property Picture: TPicture read FPicture write SetPicture;
    {$IFDEF D4}
    procedure FlipChildren(AllLevels: Boolean); override;
    {$ENDIF}
  end;

  TagControlBar = class(TagCustomControlBar)
  public
    property Canvas;
  published
    property Align;
    property AutoSize;
    property BevelInner default bvNone;
    property BevelOuter default bvLowered;
    property BevelWidth;
    property BorderWidth;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property Picture;
    property PopupMenu;
    property RowSize;
    property RowSnap;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property Vertical;
    property OnBandDrag;
    property OnBandInfo;
    property OnBandMove;
    property OnBandPaint;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnPaint;
    property OnResize;
    property OnStartDrag;
    {$IFDEF D4}
    property Anchors;
    property AutoDrag;
    property BevelEdges;
    property BevelKind;
    property Constraints;
    property DockSite;
    property DragKind;
    property OnCanResize;
    property OnConstrainedResize;
    property OnDockDrop;
    property OnDockOver;
    property OnEndDock;
    property OnGetSiteInfo;
    property OnStartDock;
    property OnUnDock;
    {$ENDIF}
  end;

implementation

uses Consts;

{ TagCustomControlBar }

type
  PDockPos = ^TDockPos;
  TDockPos = record
    Control: TControl;
    Insets: TRect;
    Visible: Boolean;
    Break: Boolean;
    Pos: TPoint;
    Width: Integer;

    Height: Integer;
    RowCount: Integer;
    TempRow: Integer;
    Parent: PDockPos;
    SubItem: PDockPos;

    TempBreak: Boolean;
    TempPos: TPoint;
    TempWidth: Integer;
  end;

function CreateDockPos(AControl: TControl; Break: Boolean; Visible: Boolean;
  const APos: TPoint; AWidth, AHeight: Integer; Parent: PDockPos;
  const Insets: TRect; RowCount: Integer): PDockPos;
begin
  GetMem(Result, SizeOf(TDockPos));
  Result.Control := AControl;
  Result.Insets := Insets;
  Result.Visible := Visible;
  Result.Break := Break;
  Result.Pos := APos;
  Result.Width := AWidth;
  Result.Height := AHeight;
  Result.RowCount := RowCount;
  Result.TempRow := 1;
  Result.TempBreak := Break;
  Result.TempPos := APos;
  Result.TempWidth := AWidth;
  Result.Parent := Parent;
  Result.SubItem := nil;
end;

procedure FreeDockPos(Items: TList; DockPos: PDockPos);
var
  Tmp: PDockPos;
begin
  { Remove all subitems }
  while DockPos <> nil do
  begin
    Tmp := DockPos;
    Items.Remove(DockPos);
    DockPos := DockPos.SubItem;
    FreeMem(Tmp, SizeOf(TDockPos));
  end;
end;

procedure AdjustControlRect(var ARect: TRect; const Insets: TRect);
begin
  with Insets do
  begin
    Dec(ARect.Left, Left);
    Dec(ARect.Top, Top);
    Inc(ARect.Right, Right);
    Inc(ARect.Bottom, Bottom);
  end;
end;

constructor TagCustomControlBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
    csDoubleClicks, csOpaque];
  Width := 100;
  Height := 50;
  FAutoDrag := True;
  FItems := TList.Create;
  FPicture := TPicture.Create;
  FPicture.OnChange := PictureChanged;
  FRowSize := 26;
  FRowSnap := True;
  BevelInner := bvNone;
  BevelOuter := bvLowered;
  {$IFDEF D4}
  BevelKind := bkTile;
  DoubleBuffered := True;
  DockSite := True;
  {$ELSE}
  FDrawBevel:=false;
  {$ENDIF}
end;

destructor TagCustomControlBar.Destroy;
var
  I: Integer;
begin
  for I := 0 to FItems.Count - 1 do
    if FItems[I] <> nil then
      FreeMem(PDockPos(FItems[I]), SizeOf(TDockPos));
  FItems.Free;
  FPicture.Free;
  inherited Destroy;
end;

procedure TagCustomControlBar.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params.WindowClass do
    style := style and not (CS_HREDRAW or CS_VREDRAW);
end;

procedure TagCustomControlBar.SetVertical(Value: Boolean);
begin
  if Value <> FVertical then begin
    FVertical:=Value;
    ResetDockItems;
  end;
end;

procedure TagCustomControlBar.AlignControls(AControl: TControl; var ARect: TRect);
var
  I, J, X: Integer;
  DockPos: PDockPos;
  TotalSize, RowSize, RowSpace, Shift: Integer;
  RowHeight, PrevRowHeight: Integer;
  MoveBy: Integer;
  Pos: TPoint;
  CX: Integer;
  Control: TControl;
  UseTemp: Boolean;

  Row: Integer;
  RowCount: Integer;
  FirstIndex, LastIndex: Integer;
  InsertingControl: Boolean;
  Dirty: Boolean;
  R: TRect;

  TempRowSize, TempRowSpace: Integer;
  AdjustX: Integer;
  DockRect: TRect;
  PreferredSize: Integer;

  TmpDockPos: PDockPos;
  Redo: PDockPos;
  RedoCount: Integer;
  SkipRedo: Boolean;

  procedure AlignControlsV;
  var
    I,J: Integer;

    function ShouldRedo(DockPos: PDockPos; const Pos: TPoint; Width: Integer): Boolean;
    begin
      { Determine whether this subitem has changed and will affect its
        parent(s). }
      if (DockPos^.Parent <> nil) and ((Pos.Y <> DockPos^.Parent^.TempPos.Y) or
        (Width <> DockPos^.Parent^.TempWidth)) then
      begin
        DockPos := DockPos^.Parent;
        { Update parents and re-perform align logic }
        repeat
          DockPos^.TempPos.Y := Pos.Y;
          DockPos^.TempWidth := Width;
          Redo := DockPos;
          DockPos := DockPos^.Parent;
        until DockPos = nil;
        Result := True;
      end
      else
        Result := False;
    end;

  begin
    {$IFNDEF D4}
    inc(ARect.Bottom,Height-ClientHeight);
    {$ENDIF}

    if FAligning then Exit;
    FAligning := True;
    try
      { Update items }
      InsertingControl := UpdateItems(AControl);
      if FItems.Count = 0 then Exit;
      RowCount := 0;
      FirstIndex := 0;
      LastIndex := FItems.Count - 1;

      { Find number of rows }
      for I := FirstIndex to LastIndex do
      begin
        DockPos := PDockPos(FItems[I]);
        { First item can't have Break set! }
        DockPos^.TempBreak := DockPos^.Break;
        if DockPos^.Break then
          Inc(RowCount);
      end;

      Redo := nil;
      SkipRedo := False;
      RedoCount := 2;
      repeat

        if Redo <> nil then
        begin
          SkipRedo := True;
          Dec(RedoCount);
        end;
        if RedoCount = 0 then Exit;

        RowHeight := 0;
        PrevRowHeight := 0;
        Row := 1;

        while Row <= RowCount do
        begin

          if Row = 1 then
            {$IFDEF D4}
            RowHeight := 0;
            {$ELSE}
            RowHeight := ClientRect.Left;
            {$ENDIF}

          { Find first and last index for current row }
          if Row = 1 then
            FirstIndex := 0
          else
            FirstIndex := LastIndex + 1;
          LastIndex := FItems.Count - 1;
          for I := FirstIndex to LastIndex - 1 do
          begin
            DockPos := PDockPos(FItems[I + 1]);
            { First item can't have Break set }
            if DockPos^.Break or DockPos^.TempBreak then
            begin
              LastIndex := I;
              Break;
            end;
          end;

          { Set temp values for all controls }
          TotalSize := ARect.Bottom - ARect.Top;
          RowSize := 0;
          RowSpace := 0;

          for I := FirstIndex to LastIndex do
          begin
            DockPos := PDockPos(FItems[I]);

            if DockPos^.Break or DockPos^.TempBreak then
            begin
              {$IFDEF D4}
              RowSize := 0;
              {$ELSE}
              RowSize := ClientRect.Top;
              {$ENDIF}
              RowSpace := 0;
              UseTemp := False;
              if UseTemp then
                DockPos^.TempPos.X := RowHeight else
                DockPos^.Pos.X := RowHeight;
              PrevRowHeight := RowHeight;
            end
            else UseTemp := False;

            Control := DockPos^.Control;
            if (csDesigning in ComponentState) or Control.Visible then
            begin
              { If control was moved/resized, update our info }
              if DockPos^.Parent = nil then
              begin
                PreferredSize := DockPos^.Height;
                Dec(PreferredSize, DockPos^.Insets.Top + DockPos^.Insets.Bottom);
                GetControlInfo(Control, DockPos^.Insets, PreferredSize,
                  DockPos^.RowCount);
                DockPos^.Height := PreferredSize + DockPos^.Insets.Top +
                  DockPos^.Insets.Bottom;
                if not InsertingControl and (DockPos^.Parent = nil) and
                  (AControl = DockPos^.Control) then
                begin
                  if UseTemp then
                  begin
                    DockPos^.TempPos := Point(AControl.Left - ARect.Left -
                      DockPos^.Insets.Left, AControl.Top - ARect.Top - DockPos^.Insets.Top);
                    DockPos^.TempWidth := AControl.Height + DockPos^.Insets.Top +
                      DockPos^.Insets.Bottom;
                    DockRect := Bounds(DockPos^.TempPos.X, DockPos^.TempPos.Y,
                      DockPos^.Width, DockPos^.TempWidth);
                  end
                  else
                    DockRect := Bounds(DockPos^.Pos.X, DockPos^.Pos.Y,
                      DockPos^.Width, DockPos^.Height);
                end;

                { Let user adjust sizes }
                if DockPos = Redo then
                  DockRect := Bounds(DockPos^.TempPos.X, DockPos^.TempPos.Y,
                    DockPos^.Width, DockPos^.TempWidth)
                else
                  DockRect := Bounds(DockPos^.Pos.X, DockPos^.Pos.Y,
                    DockPos^.Width, DockPos^.Height);
                DoBandMove(Control, DockRect);
                DockPos^.TempWidth := DockRect.Bottom - DockRect.Top;
              end
              else
              begin
                { Use parent's position }
                with DockPos^.Parent^ do
                begin
                  DockPos^.Pos := Pos;
                  DockPos^.TempPos := TempPos;
                  Inc(DockPos^.Pos.X, Width);
                  Inc(DockPos^.TempPos.X, Width);
                  DockPos^.Height := Height;
                  DockPos^.TempWidth := TempWidth;
                  DockRect := Bounds(DockPos^.TempPos.X, DockPos^.TempPos.Y,
                    DockPos^.Width, DockPos^.TempWidth);
                end;
              end;

              if DockPos = Redo then
              begin
                with DockPos^ do
                begin
                  TempPos.X := DockRect.Left;
                  TempPos.Y := DockRect.Top;
                  TempWidth := DockRect.Bottom - DockRect.Top;
                  Redo := nil;
                  SkipRedo := False;
                end;
              end
              else
              begin
                with DockPos^ do
                begin
                  Pos.X := DockRect.Left;
                  Pos.Y := DockRect.Top;
                end;
              end;

              if UseTemp then
              begin
                Pos := DockPos^.TempPos;
                CX := DockPos^.TempWidth;
              end
              else
              begin
                Pos := DockRect.TopLeft;
                CX := DockRect.Bottom - DockRect.Top;
              end;

              { Make sure Pos is within bounds }
              if Pos.Y < RowSize then
              begin
                { If a control is being resized/moved then adjust any controls to
                  its left }
                if (RowSpace > 0) then
                begin
                  TempRowSize := Pos.Y;
                  AdjustX := Pos.Y;
                  TempRowSpace := RowSpace;
                  for J := I - 1 downto FirstIndex do
                  begin
                    with PDockPos(FItems[J])^ do
                    begin
                      if (csDesigning in ComponentState) or Control.Visible then
                      begin
                        if TempPos.Y + TempWidth > TempRowSize then
                        begin
                          X := TempPos.Y + TempWidth - TempRowSize;
                          { Calculate adjusted rowspace }
                          if J < I - 1 then
                            Dec(TempRowSpace, AdjustX - (TempPos.Y + TempWidth));
                          if X > TempRowSpace then
                            X := TempRowSpace;
                          AdjustX := TempPos.Y;
                          Dec(TempPos.Y, X);
                          Dec(TempRowSize, TempWidth);

                          TmpDockPos := PDockPos(FItems[J]);
                          if ShouldRedo(TmpDockPos, TmpDockPos^.TempPos,
                            TmpDockPos^.TempWidth) then
                            System.Break;

                          TmpDockPos := SubItem;
                          while TmpDockPos <> nil do
                            with TmpDockPos^ do
                            begin
                              Pos := PDockPos(FItems[J])^.Pos;
                              TempPos := PDockPos(FItems[J])^.TempPos;
                              Inc(Pos.X, Parent.Width);
                              Inc(TempPos.X, Parent.Width);
                              Height := PDockPos(FItems[J])^.Height;
                              TempWidth := PDockPos(FItems[J])^.TempWidth;
                              TmpDockPos := SubItem;
                            end;

                        end
                        else System.Break;
                      end;
                    end;
                  end;
                  AdjustX := RowSize - Pos.Y;
                  if AdjustX > RowSpace then
                    AdjustX := RowSpace;
                  Dec(RowSpace, AdjustX);
                  Dec(RowSize, AdjustX);
                end;
                Pos.Y := RowSize;
              end;

              if (Redo <> nil) and not SkipRedo then Break;

              if Pos.X <> PrevRowHeight then
                Pos.X := PrevRowHeight;

              if Pos.X + DockPos^.Width > RowHeight then
                RowHeight := Pos.X + DockPos^.Width;

              Inc(RowSpace, Pos.Y - RowSize);
              Inc(RowSize, Pos.Y - RowSize + CX);

              if DockPos^.Parent = nil then
              begin
                DockPos^.TempPos := Pos;
                DockPos^.TempWidth := CX;
              end
              else
              begin
                if ShouldRedo(DockPos, Pos, CX) then
                  System.Break
                else if not DockPos^.Break and (DockPos^.TempPos.Y < Pos.Y) then
                begin
                  DockPos^.TempPos := Pos;
                  DockPos^.TempWidth := CX;
                end;
              end;

              TmpDockPos := DockPos^.SubItem;
              while TmpDockPos <> nil do
                with TmpDockPos^ do
                begin
                  Pos := DockPos^.Pos;
                  TempPos := DockPos^.TempPos;
                  Inc(Pos.X, Parent.Width);
                  Inc(TempPos.X, Parent.Width);
                  Height := DockPos^.Height;
                  TempWidth := DockPos^.TempWidth;
                  TmpDockPos := SubItem;
                end;
            end;
          end;

          if (Redo <> nil) and not SkipRedo then Break;

          { Determine whether controls on this row can fit }
          Shift := TotalSize - RowSize;
          if Shift < 0 then
          begin
            TotalSize := ARect.Bottom - ARect.Top;
            { Try to move all controls to fill space }
            AdjustX := RowSize;
            TempRowSpace := RowSpace;
            for I := LastIndex downto FirstIndex do
            begin
              DockPos := PDockPos(FItems[I]);
              Control := DockPos^.Control;
              if (csDesigning in ComponentState) or Control.Visible then
              begin
                if (DockPos^.TempPos.Y + DockPos^.TempWidth) > TotalSize then
                begin
                  MoveBy := (DockPos^.TempPos.Y + DockPos^.TempWidth) - TotalSize;
                  if I < LastIndex then
                    Dec(TempRowSpace, AdjustX - (DockPos^.TempPos.Y +
                      DockPos^.TempWidth));
                  if MoveBy <= TempRowSpace then
                    Shift := MoveBy else
                    Shift := TempRowSpace;
                  if Shift <= TempRowSpace then
                  begin
                    AdjustX := DockPos^.TempPos.Y;
                    Dec(DockPos^.TempPos.Y, Shift);
                    Dec(TotalSize, DockPos^.TempWidth);

                    if ShouldRedo(DockPos, DockPos^.TempPos, DockPos^.TempWidth) then
                      Break;

                    TmpDockPos := DockPos^.SubItem;
                    while TmpDockPos <> nil do
                      with TmpDockPos^ do
                      begin
                        TempPos := DockPos^.TempPos;
                        Inc(TempPos.X, Parent.Width);
                        TmpDockPos := SubItem;
                      end;
                  end
                  else
                    Break;
                end;
              end;
            end;

            if (Redo <> nil) and not SkipRedo then Break;

            { Try to minimize all controls to fill space }
            if TotalSize < 0 then
            begin
              TotalSize := ARect.Bottom - ARect.Top;
              for I := LastIndex downto FirstIndex do
              begin
                DockPos := PDockPos(FItems[I]);
                Control := DockPos^.Control;
                if (csDesigning in ComponentState) or Control.Visible then
                begin
                  if DockPos^.TempPos.Y + DockPos^.TempWidth > TotalSize then
                  begin
                    { Try to minimize control, move if it can't be resized }
                    DockPos^.TempWidth := DockPos^.TempWidth -
                      ((DockPos^.TempPos.Y + DockPos^.TempWidth) - TotalSize);
                    if DockPos^.TempWidth < {$IFDEF D4}Control.Constraints.MinHeight +{$ENDIF}
                      DockPos^.Insets.Top + DockPos^.Insets.Bottom then
                      DockPos^.TempWidth := {$IFDEF D4}Control.Constraints.MinHeight +{$ENDIF}
                        DockPos^.Insets.Top + DockPos^.Insets.Bottom;
                    { Move control }
                    if DockPos^.TempPos.Y + DockPos^.TempWidth > TotalSize then
                    begin
                      Dec(DockPos^.TempPos.Y, (DockPos^.TempPos.Y +
                        DockPos^.TempWidth) - TotalSize);
                      if DockPos^.TempPos.Y < ARect.Top then
                        DockPos^.TempPos.Y := ARect.Top;
                    end;

                    if ShouldRedo(DockPos, DockPos^.TempPos, DockPos^.TempWidth) then
                      Break;

                    TmpDockPos := DockPos^.SubItem;
                    while TmpDockPos <> nil do
                      with TmpDockPos^ do
                      begin
                        Pos := DockPos^.Pos;
                        TempPos := DockPos^.TempPos;
                        Inc(TempPos.X, Parent.Width);
                        TempWidth := DockPos^.TempWidth;
                        TmpDockPos := SubItem;
                      end;
                  end;
                  Dec(TotalSize, DockPos^.TempWidth);
                end;
              end;
            end;

            if (Redo <> nil) and not SkipRedo then Break;

            { Done with first pass at minimizing. If we're still cramped for
              space then wrap last control if there are more than 1 controls on
              this row. }
            if (TotalSize < 0) and (FirstIndex <> LastIndex) then
            begin
              DockPos := PDockPos(FItems[LastIndex]);
              DockPos^.TempPos.Y := 0;
              DockPos^.TempWidth := DockPos^.Height;
              DockPos^.TempBreak := True;
              Inc(RowCount);

              if ShouldRedo(DockPos, DockPos^.TempPos, DockPos^.TempWidth) then
                Break;

              TmpDockPos := DockPos^.SubItem;
              while TmpDockPos <> nil do
                with TmpDockPos^ do
                begin
                  TempPos := DockPos^.TempPos;
                  Inc(TempPos.X, Parent.Width);
                  TempWidth := DockPos^.TempWidth;
                  TmpDockPos := SubItem;
                end;
            end
            else
              Inc(Row);
          end
          else
            Inc(Row);

        end;

      until Redo = nil;

      { Now position controls }
      for I := 0 to FItems.Count - 1 do
      begin
        DockPos := PDockPos(FItems[I]);
        with DockPos^ do
          if (Parent = nil) and ((csDesigning in ComponentState) or
            Control.Visible) then
          begin
            with Insets do
              R := Rect(Left + TempPos.X, Top + TempPos.Y,
                TempPos.X + DockPos^.Width - Right,
                TempPos.Y + TempWidth - Bottom);
            TmpDockPos := SubItem;
            while TmpDockPos <> nil do
            begin
              Inc(R.Right, TmpDockPos^.Width);
              TmpDockPos := TmpDockPos^.SubItem;
            end;
            if (R.Left <> Control.Left) or (R.Top <> Control.Top) or
              (R.Right - R.Left <> Control.Width) or
              (R.Bottom - R.Top <> Control.Height) then
            begin
              Dirty := True;
              Control.BoundsRect := R;
            end;
          end;
      end;
      if Dirty or (AControl <> nil) then Invalidate;
      { Apply any constraints }
      AdjustSize;
    finally
      FAligning := False;
    end;
  end;

  function ShouldRedo(DockPos: PDockPos; const Pos: TPoint; Width: Integer): Boolean;
  begin
    { Determine whether this subitem has changed and will affect its
      parent(s). }
    if (DockPos^.Parent <> nil) and ((Pos.X <> DockPos^.Parent^.TempPos.X) or
      (Width <> DockPos^.Parent^.TempWidth)) then
    begin
      DockPos := DockPos^.Parent;
      { Update parents and re-perform align logic }
      repeat
        DockPos^.TempPos.X := Pos.X;
        DockPos^.TempWidth := Width;
        Redo := DockPos;
        DockPos := DockPos^.Parent;
      until DockPos = nil;
      Result := True;
    end
    else
      Result := False;
  end;

begin
  if FVertical then begin
    AlignControlsV;
    Exit;
  end;

  {$IFNDEF D4}
  inc(ARect.Right,Width-ClientWidth);
  {$ENDIF}

  if FAligning then Exit;
  FAligning := True;
  try
    { Update items }
    InsertingControl := UpdateItems(AControl);
    if FItems.Count = 0 then Exit;
    RowCount := 0;
    FirstIndex := 0;
    LastIndex := FItems.Count - 1;

    { Find number of rows }
    for I := FirstIndex to LastIndex do
    begin
      DockPos := PDockPos(FItems[I]);
      { First item can't have Break set! }
      DockPos^.TempBreak := DockPos^.Break;
      if DockPos^.Break then
        Inc(RowCount);
    end;

    Redo := nil;
    SkipRedo := False;
    RedoCount := 2;
    repeat

      if Redo <> nil then
      begin
        SkipRedo := True;
        Dec(RedoCount);
      end;
      if RedoCount = 0 then Exit;

      RowHeight := 0;
      PrevRowHeight := 0;
      Row := 1;

      while Row <= RowCount do
      begin

        if Row = 1 then
          {$IFDEF D4}
          RowHeight := 0;
          {$ELSE}
          RowHeight := ClientRect.Top;
          {$ENDIF}

        { Find first and last index for current row }
        if Row = 1 then
          FirstIndex := 0
        else
          FirstIndex := LastIndex + 1;
        LastIndex := FItems.Count - 1;
        for I := FirstIndex to LastIndex - 1 do
        begin
          DockPos := PDockPos(FItems[I + 1]);
          { First item can't have Break set }
          if DockPos^.Break or DockPos^.TempBreak then
          begin
            LastIndex := I;
            Break;
          end;
        end;

        { Set temp values for all controls }
        TotalSize := ARect.Right - ARect.Left;
        RowSize := 0;
        RowSpace := 0;

        for I := FirstIndex to LastIndex do
        begin
          DockPos := PDockPos(FItems[I]);

          if DockPos^.Break or DockPos^.TempBreak then
          begin
            {$IFDEF D4}
            RowSize := 0;
            {$ELSE}
            RowSize := ClientRect.Left;
            {$ENDIF}
            RowSpace := 0;
            UseTemp := False;
            if UseTemp then
              DockPos^.TempPos.Y := RowHeight else
              DockPos^.Pos.Y := RowHeight;
            PrevRowHeight := RowHeight;
          end
          else UseTemp := False;

          Control := DockPos^.Control;
          if (csDesigning in ComponentState) or Control.Visible then
          begin
            { If control was moved/resized, update our info }
            if DockPos^.Parent = nil then
            begin
              PreferredSize := DockPos^.Width;
              Dec(PreferredSize, DockPos^.Insets.Left + DockPos^.Insets.Right);
              GetControlInfo(Control, DockPos^.Insets, PreferredSize,
                DockPos^.RowCount);
              DockPos^.Width := PreferredSize + DockPos^.Insets.Left +
                DockPos^.Insets.Right;
              if not InsertingControl and (DockPos^.Parent = nil) and
                (AControl = DockPos^.Control) then
              begin
                if UseTemp then
                begin
                  DockPos^.TempPos := Point(AControl.Left - ARect.Left -
                    DockPos^.Insets.Left, AControl.Top - ARect.Top - DockPos^.Insets.Top);
                  DockPos^.TempWidth := AControl.Width + DockPos^.Insets.Left +
                    DockPos^.Insets.Right;
                  DockRect := Bounds(DockPos^.TempPos.X, DockPos^.TempPos.Y,
                    DockPos^.TempWidth, DockPos^.Height);
                end
                else
                  DockRect := Bounds(DockPos^.Pos.X, DockPos^.Pos.Y,
                    DockPos^.Width, DockPos^.Height);
              end;

              { Let user adjust sizes }
              if DockPos = Redo then
                DockRect := Bounds(DockPos^.TempPos.X, DockPos^.TempPos.Y,
                  DockPos^.TempWidth, DockPos^.Height)
              else
                DockRect := Bounds(DockPos^.Pos.X, DockPos^.Pos.Y,
                  DockPos^.Width, DockPos^.Height);
              DoBandMove(Control, DockRect);
              DockPos^.TempWidth := DockRect.Right - DockRect.Left;
            end
            else
            begin
              { Use parent's position }
              with DockPos^.Parent^ do
              begin
                DockPos^.Pos := Pos;
                DockPos^.TempPos := TempPos;
                Inc(DockPos^.Pos.Y, Height);
                Inc(DockPos^.TempPos.Y, Height);
                DockPos^.Width := Width;
                DockPos^.TempWidth := TempWidth;
                DockRect := Bounds(DockPos^.TempPos.X, DockPos^.TempPos.Y,
                  DockPos^.TempWidth, DockPos^.Height);
              end;
            end;

            if DockPos = Redo then
            begin
              with DockPos^ do
              begin
                TempPos.X := DockRect.Left;
                TempPos.Y := DockRect.Top;
                TempWidth := DockRect.Right - DockRect.Left;
                Redo := nil;
                SkipRedo := False;
              end;
            end
            else
            begin
              with DockPos^ do
              begin
                Pos.X := DockRect.Left;
                Pos.Y := DockRect.Top;
              end;
            end;

            if UseTemp then
            begin
              Pos := DockPos^.TempPos;
              CX := DockPos^.TempWidth;
            end
            else
            begin
              Pos := DockRect.TopLeft;
              CX := DockRect.Right - DockRect.Left;
            end;

            { Make sure Pos is within bounds }
            if Pos.X < RowSize then
            begin
              { If a control is being resized/moved then adjust any controls to
                its left }
              if (RowSpace > 0) then
              begin
                TempRowSize := Pos.X;
                AdjustX := Pos.X;
                TempRowSpace := RowSpace;
                for J := I - 1 downto FirstIndex do
                begin
                  with PDockPos(FItems[J])^ do
                  begin
                    if (csDesigning in ComponentState) or Control.Visible then
                    begin
                      if TempPos.X + TempWidth > TempRowSize then
                      begin
                        X := TempPos.X + TempWidth - TempRowSize;
                        { Calculate adjusted rowspace }
                        if J < I - 1 then
                          Dec(TempRowSpace, AdjustX - (TempPos.X + TempWidth));
                        if X > TempRowSpace then
                          X := TempRowSpace;
                        AdjustX := TempPos.X;
                        Dec(TempPos.X, X);
                        Dec(TempRowSize, TempWidth);

                        TmpDockPos := PDockPos(FItems[J]);
                        if ShouldRedo(TmpDockPos, TmpDockPos^.TempPos,
                          TmpDockPos^.TempWidth) then
                          System.Break;

                        TmpDockPos := SubItem;
                        while TmpDockPos <> nil do
                          with TmpDockPos^ do
                          begin
                            Pos := PDockPos(FItems[J])^.Pos;
                            TempPos := PDockPos(FItems[J])^.TempPos;
                            Inc(Pos.Y, Parent.Height);
                            Inc(TempPos.Y, Parent.Height);
                            Width := PDockPos(FItems[J])^.Width;
                            TempWidth := PDockPos(FItems[J])^.TempWidth;
                            TmpDockPos := SubItem;
                          end;

                      end
                      else System.Break;
                    end;
                  end;
                end;
                AdjustX := RowSize - Pos.X;
                if AdjustX > RowSpace then
                  AdjustX := RowSpace;
                Dec(RowSpace, AdjustX);
                Dec(RowSize, AdjustX);
              end;
              Pos.X := RowSize;
            end;

            if (Redo <> nil) and not SkipRedo then Break;

            if Pos.Y <> PrevRowHeight then
              Pos.Y := PrevRowHeight;

            if Pos.Y + DockPos^.Height > RowHeight then
              RowHeight := Pos.Y + DockPos^.Height;

            Inc(RowSpace, Pos.X - RowSize);
            Inc(RowSize, Pos.X - RowSize + CX);

            if DockPos^.Parent = nil then
            begin
              DockPos^.TempPos := Pos;
              DockPos^.TempWidth := CX;
            end
            else
            begin
              if ShouldRedo(DockPos, Pos, CX) then
                System.Break
              else if not DockPos^.Break and (DockPos^.TempPos.X < Pos.X) then
              begin
                DockPos^.TempPos := Pos;
                DockPos^.TempWidth := CX;
              end;
            end;

            TmpDockPos := DockPos^.SubItem;
            while TmpDockPos <> nil do
              with TmpDockPos^ do
              begin
                Pos := DockPos^.Pos;
                TempPos := DockPos^.TempPos;
                Inc(Pos.Y, Parent.Height);
                Inc(TempPos.Y, Parent.Height);
                Width := DockPos^.Width;
                TempWidth := DockPos^.TempWidth;
                TmpDockPos := SubItem;
              end;
          end;
        end;

        if (Redo <> nil) and not SkipRedo then Break;

        { Determine whether controls on this row can fit }
        Shift := TotalSize - RowSize;
        if Shift < 0 then
        begin
          TotalSize := ARect.Right - ARect.Left;
          { Try to move all controls to fill space }
          AdjustX := RowSize;
          TempRowSpace := RowSpace;
          for I := LastIndex downto FirstIndex do
          begin
            DockPos := PDockPos(FItems[I]);
            Control := DockPos^.Control;
            if (csDesigning in ComponentState) or Control.Visible then
            begin
              if (DockPos^.TempPos.X + DockPos^.TempWidth) > TotalSize then
              begin
                MoveBy := (DockPos^.TempPos.X + DockPos^.TempWidth) - TotalSize;
                if I < LastIndex then
                  Dec(TempRowSpace, AdjustX - (DockPos^.TempPos.X +
                    DockPos^.TempWidth));
                if MoveBy <= TempRowSpace then
                  Shift := MoveBy else
                  Shift := TempRowSpace;
                if Shift <= TempRowSpace then
                begin
                  AdjustX := DockPos^.TempPos.X;
                  Dec(DockPos^.TempPos.X, Shift);
                  Dec(TotalSize, DockPos^.TempWidth);

                  if ShouldRedo(DockPos, DockPos^.TempPos, DockPos^.TempWidth) then
                    Break;

                  TmpDockPos := DockPos^.SubItem;
                  while TmpDockPos <> nil do
                    with TmpDockPos^ do
                    begin
                      TempPos := DockPos^.TempPos;
                      Inc(TempPos.Y, Parent.Height);
                      TmpDockPos := SubItem;
                    end;
                end
                else
                  Break;
              end;
            end;
          end;

          if (Redo <> nil) and not SkipRedo then Break;

          { Try to minimize all controls to fill space }
          if TotalSize < 0 then
          begin
            TotalSize := ARect.Right - ARect.Left;
            for I := LastIndex downto FirstIndex do
            begin
              DockPos := PDockPos(FItems[I]);
              Control := DockPos^.Control;
              if (csDesigning in ComponentState) or Control.Visible then
              begin
                if DockPos^.TempPos.X + DockPos^.TempWidth > TotalSize then
                begin
                  { Try to minimize control, move if it can't be resized }
                  DockPos^.TempWidth := DockPos^.TempWidth -
                    ((DockPos^.TempPos.X + DockPos^.TempWidth) - TotalSize);
                  if DockPos^.TempWidth < {$IFDEF D4}Control.Constraints.MinWidth +{$ENDIF}
                    DockPos^.Insets.Left + DockPos^.Insets.Right then
                    DockPos^.TempWidth := {$IFDEF D4}Control.Constraints.MinWidth +{$ENDIF}
                      DockPos^.Insets.Left + DockPos^.Insets.Right;
                  { Move control }
                  if DockPos^.TempPos.X + DockPos^.TempWidth > TotalSize then
                  begin
                    Dec(DockPos^.TempPos.X, (DockPos^.TempPos.X +
                      DockPos^.TempWidth) - TotalSize);
                    if DockPos^.TempPos.X < ARect.Left then
                      DockPos^.TempPos.X := ARect.Left;
                  end;

                  if ShouldRedo(DockPos, DockPos^.TempPos, DockPos^.TempWidth) then
                    Break;

                  TmpDockPos := DockPos^.SubItem;
                  while TmpDockPos <> nil do
                    with TmpDockPos^ do
                    begin
                      Pos := DockPos^.Pos;
                      TempPos := DockPos^.TempPos;
                      Inc(TempPos.Y, Parent.Height);
                      TempWidth := DockPos^.TempWidth;
                      TmpDockPos := SubItem;
                    end;
                end;
                Dec(TotalSize, DockPos^.TempWidth);
              end;
            end;
          end;

          if (Redo <> nil) and not SkipRedo then Break;

          { Done with first pass at minimizing. If we're still cramped for
            space then wrap last control if there are more than 1 controls on
            this row. }
          if (TotalSize < 0) and (FirstIndex <> LastIndex) then
          begin
            DockPos := PDockPos(FItems[LastIndex]);
            DockPos^.TempPos.X := 0;
            DockPos^.TempWidth := DockPos^.Width;
            DockPos^.TempBreak := True;
            Inc(RowCount);

            if ShouldRedo(DockPos, DockPos^.TempPos, DockPos^.TempWidth) then
              Break;

            TmpDockPos := DockPos^.SubItem;
            while TmpDockPos <> nil do
              with TmpDockPos^ do
              begin
                TempPos := DockPos^.TempPos;
                Inc(TempPos.Y, Parent.Height);
                TempWidth := DockPos^.TempWidth;
                TmpDockPos := SubItem;
              end;
          end
          else
            Inc(Row);
        end
        else
          Inc(Row);

      end;

    until Redo = nil;

    { Now position controls }
    for I := 0 to FItems.Count - 1 do
    begin
      DockPos := PDockPos(FItems[I]);
      with DockPos^ do
        if (Parent = nil) and ((csDesigning in ComponentState) or
          Control.Visible) then
        begin
          with Insets do
            R := Rect(Left + TempPos.X, Top + TempPos.Y,
              TempPos.X + TempWidth - Right,
              TempPos.Y + DockPos^.Height - Bottom);
          TmpDockPos := SubItem;
          while TmpDockPos <> nil do
          begin
            Inc(R.Bottom, TmpDockPos^.Height);
            TmpDockPos := TmpDockPos^.SubItem;
          end;
          if (R.Left <> Control.Left) or (R.Top <> Control.Top) or
            (R.Right - R.Left <> Control.Width) or
            (R.Bottom - R.Top <> Control.Height) then
          begin
            Dirty := True;
            Control.BoundsRect := R;
          end;
        end;
    end;
    if Dirty or (AControl <> nil) then Invalidate;
    { Apply any constraints }
    AdjustSize;
  finally
    FAligning := False;
  end;
end;

const
  DefaultInsets: TRect = (Left: 11; Top: 2; Right: 2; Bottom: 2);
  DefaultInsetsV: TRect = (Left: 2; Top: 11; Right: 2; Bottom: 2);

function TagCustomControlBar.UpdateItems(AControl: TControl): Boolean;
var
  I, J, Tmp, RepositionIndex: Integer;
  PrevBreak: Boolean;
  Control: TControl;
  Exists: Boolean;
  AddControls: TList;
  DockRect, R: TRect;
  Dirty: Boolean;
  IsVisible: Boolean;
  DockPos, TmpDockPos1, TmpDockPos2: PDockPos;
  BreakList: TList;
  IndexList: TList;
  SizeList: TList;
  ChangedPriorBreak: Boolean;

  function UpdateItemsV: Boolean;
  var
    I, J: Integer;

    procedure AddControl(List: TList; Control: TControl);
    var
       I: Integer;
    begin
      for I := 0 to List.Count - 1 do
        with TControl(List[I]) do
          if (Control.Left < Left) or (Control.Left = Left) and
            (Control.Top < Top) then
          begin
            List.Insert(I, Control);
            Exit;
          end;
      List.Add(Control);
    end;

  begin
    Result := False;
    ChangedPriorBreak := False;
    AddControls := TList.Create;
    BreakList := TList.Create;
    IndexList := TList.Create;
    SizeList := TList.Create;
    try
      AddControls.Capacity := ControlCount;
      RepositionIndex := -1;
      Dirty := False;
      for I := 0 to ControlCount - 1 do
      begin
        Control := Controls[I];
        IsVisible := (csDesigning in ComponentState) or Control.Visible;
        Exists := False;
        for J := 0 to FItems.Count - 1 do
          if (PDockPos(FItems[J])^.Parent = nil) and
            (PDockPos(FItems[J])^.Control = Control) then
          begin
            Dirty := Dirty or PDockPos(FItems[J])^.Visible <> IsVisible;
            PDockPos(FItems[J])^.Visible := IsVisible;
            Exists := True;
            Break;
          end;
        if Exists and (Control = AControl) then
        begin
          RepositionIndex := J;
          DockPos := PDockPos(FItems[J]);
          with DockPos^ do
          begin
            SizeList.Add(TObject(Insets.Left + Insets.Right));
            if FDragControl <> nil then
              DockRect := Rect(Pos.X + Insets.Left, Pos.Y + Insets.Top,
                Pos.X + Insets.Left + Control.Width, Pos.Y + Height - Insets.Bottom)
            else
              DockRect := Control.BoundsRect;
            PrevBreak := Break;
          end;
          { If we were starting a row, then update any items to the right to
            begin starting the row. }
          if PrevBreak and (J + 1 < FItems.Count) then
          begin
            TmpDockPos1 := FItems[J + 1];
            if not TmpDockPos1.Break then
            begin
              TmpDockPos1.Break := True;
              TmpDockPos1.TempBreak := True;
              ChangedPriorBreak := True;
            end;
          end;

          { Remember the state of this item and its subitems }
          BreakList.Add(TObject(Ord(PrevBreak)));
          IndexList.Add(TObject(J));
          TmpDockPos1 := DockPos^.SubItem;
          while TmpDockPos1 <> nil do
          begin
            Tmp := FItems.IndexOf(TmpDockPos1);
            BreakList.Add(TObject(Ord(TmpDockPos1.Break)));
            IndexList.Add(TObject(Tmp));
            with TmpDockPos1^ do
              SizeList.Add(TObject(Insets.Left + Insets.Right));
            { If we were starting a row, then update any items to the right to
              begin starting the row. }
            if TmpDockPos1.Break then
            begin
              if Tmp + 1 < FItems.Count then
              begin
                TmpDockPos2 := FItems[Tmp + 1];
                if not TmpDockPos2.Break then
                  TmpDockPos2.Break := True;
              end;
            end;
            TmpDockPos1 := TmpDockPos1^.SubItem;
          end;

          { Remove this item from consideration in DockControl. It's as if we are
            adding a new control. }
          FreeDockPos(FItems, DockPos);
        end
        else if not Exists then
        begin
          if Control = AControl then Result := True;
          AddControl(AddControls, Control);
        end;
      end;
      for I := 0 to AddControls.Count - 1 do
      begin
        R := TControl(AddControls[I]).BoundsRect;
        DockControl(TControl(AddControls[I]), R, BreakList, IndexList, SizeList,
          nil, ChangedPriorBreak, DefaultInsetsV, -1, -1, False);
      end;
      if RepositionIndex >= 0 then
        DockControl(AControl, DockRect, BreakList, IndexList, SizeList, nil,
          ChangedPriorBreak, DefaultInsetsV, -1, -1, True);
      if Dirty then Invalidate;
    finally
      AddControls.Free;
      BreakList.Free;
      IndexList.Free;
      SizeList.Free;
    end;
  end;

  procedure AddControl(List: TList; Control: TControl);
  var
     I: Integer;
  begin
    for I := 0 to List.Count - 1 do
      with TControl(List[I]) do
        if (Control.Top < Top) or (Control.Top = Top) and
          (Control.Left < Left) then
        begin
          List.Insert(I, Control);
          Exit;
        end;
    List.Add(Control);
  end;

begin
  if FVertical then begin
    Result:=UpdateItemsV;
    exit;
  end;

  Result := False;
  ChangedPriorBreak := False;
  AddControls := TList.Create;
  BreakList := TList.Create;
  IndexList := TList.Create;
  SizeList := TList.Create;
  try
    AddControls.Capacity := ControlCount;
    RepositionIndex := -1;
    Dirty := False;
    for I := 0 to ControlCount - 1 do
    begin
      Control := Controls[I];
      IsVisible := (csDesigning in ComponentState) or Control.Visible;
      Exists := False;
      for J := 0 to FItems.Count - 1 do
        if (PDockPos(FItems[J])^.Parent = nil) and
          (PDockPos(FItems[J])^.Control = Control) then
        begin
          Dirty := Dirty or PDockPos(FItems[J])^.Visible <> IsVisible;
          PDockPos(FItems[J])^.Visible := IsVisible;
          Exists := True;
          Break;
        end;
      if Exists and (Control = AControl) then
      begin
        RepositionIndex := J;
        DockPos := PDockPos(FItems[J]);
        with DockPos^ do
        begin
          SizeList.Add(TObject(Insets.Top + Insets.Bottom));
          if FDragControl <> nil then
            DockRect := Rect(Pos.X + Insets.Left, Pos.Y + Insets.Top,
              Pos.X + Width - Insets.Right, Pos.Y + Insets.Top + Control.Height)
          else
            DockRect := Control.BoundsRect;
          PrevBreak := Break;
        end;
        { If we were starting a row, then update any items to the right to
          begin starting the row. }
        if PrevBreak and (J + 1 < FItems.Count) then
        begin
          TmpDockPos1 := FItems[J + 1];
          if not TmpDockPos1.Break then
          begin
            TmpDockPos1.Break := True;
            TmpDockPos1.TempBreak := True;
            ChangedPriorBreak := True;
          end;
        end;

        { Remember the state of this item and its subitems }
        BreakList.Add(TObject(Ord(PrevBreak)));
        IndexList.Add(TObject(J));
        TmpDockPos1 := DockPos^.SubItem;
        while TmpDockPos1 <> nil do
        begin
          Tmp := FItems.IndexOf(TmpDockPos1);
          BreakList.Add(TObject(Ord(TmpDockPos1.Break)));
          IndexList.Add(TObject(Tmp));
          with TmpDockPos1^ do
            SizeList.Add(TObject(Insets.Top + Insets.Bottom));
          { If we were starting a row, then update any items to the right to
            begin starting the row. }
          if TmpDockPos1.Break then
          begin
            if Tmp + 1 < FItems.Count then
            begin
              TmpDockPos2 := FItems[Tmp + 1];
              if not TmpDockPos2.Break then
                TmpDockPos2.Break := True;
            end;
          end;
          TmpDockPos1 := TmpDockPos1^.SubItem;
        end;

        { Remove this item from consideration in DockControl. It's as if we are
          adding a new control. }
        FreeDockPos(FItems, DockPos);
      end
      else if not Exists then
      begin
        if Control = AControl then Result := True;
        AddControl(AddControls, Control);
      end;
    end;
    for I := 0 to AddControls.Count - 1 do
    begin
      R := TControl(AddControls[I]).BoundsRect;
      DockControl(TControl(AddControls[I]), R, BreakList, IndexList, SizeList,
        nil, ChangedPriorBreak, DefaultInsets, -1, -1, False);
    end;
    if RepositionIndex >= 0 then
      DockControl(AControl, DockRect, BreakList, IndexList, SizeList, nil,
        ChangedPriorBreak, DefaultInsets, -1, -1, True);
    if Dirty then Invalidate;
  finally
    AddControls.Free;
    BreakList.Free;
    IndexList.Free;
    SizeList.Free;
  end;
end;

procedure TagCustomControlBar.SetRowSize(Value: TagRowSize);
begin
  if Value <> RowSize then
  begin
    FRowSize := Value;
  end;
end;

procedure TagCustomControlBar.SetRowSnap(Value: Boolean);
begin
  if Value <> RowSnap then
  begin
    FRowSnap := Value;
  end;
end;

{$IFDEF D4}
procedure TagCustomControlBar.FlipChildren(AllLevels: Boolean);
begin
  { Do not flip controls }
end;
{$ENDIF}

procedure TagCustomControlBar.StickControls;
var
  I: Integer;

  procedure StickControlsV;
  var
    I: Integer;
  begin
    for I := 0 to FItems.Count - 1 do
      if FItems[I] <> nil then
        with PDockPos(FItems[I])^ do
        begin
          if Parent <> nil then
            Pos := Point(Parent^.Pos.X + Parent.Width, Parent^.Pos.Y)
          else
          begin
            Pos := Control.BoundsRect.TopLeft;
            Dec(Pos.X, Insets.Left);
            Dec(Pos.Y, Insets.Top);
          end;
          Height := Control.Height + Insets.Top + Insets.Bottom;
          Break := TempBreak;
        end;
  end;

begin
  if FVertical then begin
    StickControlsV;
    exit;
  end;

  for I := 0 to FItems.Count - 1 do
    if FItems[I] <> nil then
      with PDockPos(FItems[I])^ do
      begin
        if Parent <> nil then
          Pos := Point(Parent^.Pos.X, Parent^.Pos.Y + Parent.Height)
        else
        begin
          Pos := Control.BoundsRect.TopLeft;
          Dec(Pos.X, Insets.Left);
          Dec(Pos.Y, Insets.Top);
        end;
        Width := Control.Width + Insets.Left + Insets.Right;
        Break := TempBreak;
      end;
end;

function TagCustomControlBar.CanAutoSize(var NewWidth, NewHeight: Integer): Boolean;
var
  I: Integer;
  DockPos: PDockPos;
begin
  Result := True;
  if HandleAllocated and (not (csDesigning in ComponentState) or
    (ControlCount > 0)) then
  begin
    if Align in [alLeft, alRight] then
    begin
      NewWidth := 0;
      for I := 0 to FItems.Count - 1 do
      begin
        DockPos := PDockPos(FItems[I]);
        with DockPos^ do
        begin
          if (Parent = nil) and ((csDesigning in ComponentState) or Control.Visible) then
          begin
            if TempPos.X + Control.Width + Insets.Left + Insets.Right > NewWidth then
              NewWidth := TempPos.X + Control.Width + Insets.Left + Insets.Right;
          end;
        end;
      end;
      Inc(NewWidth, Width - ClientWidth);
    end
    else
    begin
      NewHeight := 0;
      for I := 0 to FItems.Count - 1 do
      begin
        DockPos := PDockPos(FItems[I]);
        with DockPos^ do
        begin
          if (Parent = nil) and ((csDesigning in ComponentState) or Control.Visible) then
          begin
            if TempPos.Y + Control.Height + Insets.Top + Insets.Bottom > NewHeight then
              NewHeight := TempPos.Y + Control.Height + Insets.Top + Insets.Bottom;
          end;
        end;
      end;
      Inc(NewHeight, Height - ClientHeight);
    end;
  end;
end;

procedure TagCustomControlBar.DockControl(AControl: TControl;
  const ARect: TRect; BreakList, IndexList, SizeList: TList; Parent: Pointer;
  ChangedPriorBreak: Boolean; Insets: TRect; PreferredSize, RowCount: Integer;
  Existing: Boolean);
var
  I, InsPos, Size, TotalSize: Integer;
  DockPos: PDockPos;
  MidPoint: TPoint;
  NewControlRect, ControlRect: TRect;
  IsVisible, DockBreak: Boolean;
  PrevBreak: Boolean;
  PrevIndex: Integer;
  NewHeight, PrevInsetHeight: Integer;
  NewLine: Boolean;

  procedure DockControlV;
  var
    I: Integer;

    procedure AddItem;
    var
      DockPos: PDockPos;
      H: Integer;
    begin
      if InsPos = 0 then DockBreak := True;
      if (PrevIndex <> InsPos) or ChangedPriorBreak then
      begin
        if DockBreak and (InsPos < FItems.Count) then
        begin
          DockPos := FItems[InsPos];
          if not NewLine and DockPos^.Break then
          begin
            DockPos^.Break := False;
            DockPos^.TempBreak := False;
          end;
        end;
      end;
      if RowSnap then
        H := RowSize else
        H := NewControlRect.Right - NewControlRect.Left;
      DockPos := CreateDockPos(AControl, DockBreak, IsVisible,
        NewControlRect.TopLeft, H, NewControlRect.Bottom - NewControlRect.Top,
        Parent, Insets, RowCount);
      if Parent <> nil then
        PDockPos(Parent).SubItem := DockPos;
      FItems.Insert(InsPos, DockPos);
      { If we're adding an item that spans more than one row, we need to add
        pseudo items which are linked to this item. }
      if RowCount > 1 then
      begin
        Dec(RowCount);
        Inc(NewControlRect.Left, RowSize);
        DockControl(AControl, NewControlRect, BreakList, IndexList, SizeList,
          DockPos, False, Insets, PreferredSize, RowCount, False);
      end;
    end;

  begin
    FDockingControl := AControl;
    if BreakList.Count > 0 then
    begin
      PrevBreak := Boolean(BreakList[0]);
      BreakList.Delete(0);
    end
    else
      PrevBreak := False;
    if IndexList.Count > 0 then
    begin
      PrevIndex := Integer(IndexList[0]);
      IndexList.Delete(0);
    end
    else
      PrevIndex := -1;
    if SizeList.Count > 0 then
    begin
      PrevInsetHeight := Integer(SizeList[0]);
      SizeList.Delete(0);
    end
    else
      PrevInsetHeight := -1;

    InsPos := 0;
    Size := -MaxInt;
    TotalSize := -MaxInt;

    NewControlRect := ARect;
    if RowCount < 0 then
      with AControl do
      begin
        PreferredSize := ARect.Bottom - ARect.Top;
        Insets := DefaultInsetsV;
        if PrevInsetHeight < 0 then
          PrevInsetHeight := Insets.Left + Insets.Right;
        { Try to fit control into row size }
        NewHeight := PrevInsetHeight + NewControlRect.Right - NewControlRect.Left;
        if RowSnap then
        begin
          RowCount := NewHeight div RowSize;
          if RowCount = 0 then
            Inc(RowCount);
          if Existing and (NewHeight > RowSize * RowCount) then
            Inc(RowCount);
        end
        else
          RowCount := 1;
        GetControlInfo(AControl, Insets, PreferredSize, RowCount);
        if RowCount = 0 then RowCount := 1;
        if RowSnap and Existing and (NewHeight > RowSize * RowCount) then
          RowCount := NewHeight div RowSize + 1;
        NewControlRect.Bottom := NewControlRect.Top + PreferredSize;
        AdjustControlRect(NewControlRect, Insets);
        if RowSnap then
          NewControlRect.Right := NewControlRect.Left + RowSize * RowCount;
      end;

    IsVisible := (csDesigning in Self.ComponentState) or AControl.Visible;
    MidPoint.X := NewControlRect.Left + RowSize div 2;
    DockBreak := False;
    NewLine := False;

    for I := 0 to FItems.Count - 1 do
    begin
      DockPos := PDockPos(FItems[I]);
      ControlRect := Rect(DockPos^.Pos.X, DockPos^.Pos.Y, DockPos^.Pos.X +
        DockPos^.Width, DockPos^.Pos.Y + DockPos^.Height );
      with ControlRect do
      begin
        if Right - Left > Size then
          Size := Right - Left;
        if Right > TotalSize then
          TotalSize := Right;

        if (NewControlRect.Top > Top) and (MidPoint.X > Left) then
        begin
          DockBreak := False;
          InsPos := I + 1;
        end;
      end;

      if (I = FItems.Count - 1) or ((I + 1 = PrevIndex) and (PrevBreak)) or
        PDockPos(FItems[I + 1])^.Break then
      begin
        if MidPoint.X < TotalSize then
        begin
          NewLine := (InsPos = 0) and (MidPoint.X < ControlRect.Left);
          AddItem;
          Exit;
        end
        else
        begin
          DockBreak := (ControlRect.Top > NewControlRect.Top) or
            ((DockPos^.SubItem = nil));
          InsPos := I + 1;
        end;
        if RowSnap then
          Size := RowSize else
          Size := 0;
      end;
    end;
    AddItem;
  end;

  procedure AddItem;
  var
    DockPos: PDockPos;
    H: Integer;
  begin
    if InsPos = 0 then DockBreak := True;
    if (PrevIndex <> InsPos) or ChangedPriorBreak then
    begin
      if DockBreak and (InsPos < FItems.Count) then
      begin
        DockPos := FItems[InsPos];
        if not NewLine and DockPos^.Break then
        begin
          DockPos^.Break := False;
          DockPos^.TempBreak := False;
        end;
      end;
    end;
    if RowSnap then
      H := RowSize else
      H := NewControlRect.Bottom - NewControlRect.Top;
    DockPos := CreateDockPos(AControl, DockBreak, IsVisible,
      NewControlRect.TopLeft, NewControlRect.Right - NewControlRect.Left,
      H, Parent, Insets, RowCount);
    if Parent <> nil then
      PDockPos(Parent).SubItem := DockPos;
    FItems.Insert(InsPos, DockPos);
    { If we're adding an item that spans more than one row, we need to add
      pseudo items which are linked to this item. }
    if RowCount > 1 then
    begin
      Dec(RowCount);
      Inc(NewControlRect.Top, RowSize);
      DockControl(AControl, NewControlRect, BreakList, IndexList, SizeList,
        DockPos, False, Insets, PreferredSize, RowCount, False);
    end;
  end;

begin
  if FVertical then begin
    DockControlV;
    exit;
  end;

  FDockingControl := AControl;
  if BreakList.Count > 0 then
  begin
    PrevBreak := Boolean(BreakList[0]);
    BreakList.Delete(0);
  end
  else
    PrevBreak := False;
  if IndexList.Count > 0 then
  begin
    PrevIndex := Integer(IndexList[0]);
    IndexList.Delete(0);
  end
  else
    PrevIndex := -1;
  if SizeList.Count > 0 then
  begin
    PrevInsetHeight := Integer(SizeList[0]);
    SizeList.Delete(0);
  end
  else
    PrevInsetHeight := -1;

  InsPos := 0;
  Size := -MaxInt;
  TotalSize := -MaxInt;

  NewControlRect := ARect;
  if RowCount < 0 then
    with AControl do
    begin
      PreferredSize := ARect.Right - ARect.Left;
      Insets := DefaultInsets;
      if PrevInsetHeight < 0 then
        PrevInsetHeight := Insets.Top + Insets.Bottom;
      { Try to fit control into row size }
      NewHeight := PrevInsetHeight + NewControlRect.Bottom - NewControlRect.Top;
      if RowSnap then
      begin
        RowCount := NewHeight div RowSize;
        if RowCount = 0 then
          Inc(RowCount);
        if Existing and (NewHeight > RowSize * RowCount) then
          Inc(RowCount);
      end
      else
        RowCount := 1;
      GetControlInfo(AControl, Insets, PreferredSize, RowCount);
      if RowCount = 0 then RowCount := 1;
      if RowSnap and Existing and (NewHeight > RowSize * RowCount) then
        RowCount := NewHeight div RowSize + 1;
      NewControlRect.Right := NewControlRect.Left + PreferredSize;
      AdjustControlRect(NewControlRect, Insets);
      if RowSnap then
        NewControlRect.Bottom := NewControlRect.Top + RowSize * RowCount;
    end;

  IsVisible := (csDesigning in Self.ComponentState) or AControl.Visible;
  MidPoint.Y := NewControlRect.Top + RowSize div 2;
  DockBreak := False;
  NewLine := False;

  for I := 0 to FItems.Count - 1 do
  begin
    DockPos := PDockPos(FItems[I]);
    ControlRect := Rect(DockPos^.Pos.X, DockPos^.Pos.Y, DockPos^.Pos.X +
      DockPos^.Width, DockPos^.Pos.Y + DockPos^.Height );
    with ControlRect do
    begin
      if Bottom - Top > Size then
        Size := Bottom - Top;
      if Bottom > TotalSize then
        TotalSize := Bottom;

      if (NewControlRect.Left > Left) and (MidPoint.Y > Top) then
      begin
        DockBreak := False;
        InsPos := I + 1;
      end;
    end;

    if (I = FItems.Count - 1) or ((I + 1 = PrevIndex) and (PrevBreak)) or
      PDockPos(FItems[I + 1])^.Break then
    begin
      if MidPoint.Y < TotalSize then
      begin
        NewLine := (InsPos = 0) and (MidPoint.Y < ControlRect.Top);
        AddItem;
        Exit;
      end
      else
      begin
        DockBreak := (ControlRect.Left > NewControlRect.Left) or
          ((DockPos^.SubItem = nil));
        InsPos := I + 1;
      end;
      if RowSnap then
        Size := RowSize else
        Size := 0;
    end;
  end;
  AddItem;
end;

procedure TagCustomControlBar.UnDockControl(AControl: TControl);
var
  I: Integer;
  DockPos: PDockPos;
begin
  FDockingControl := AControl;
  for I := 0 to FItems.Count - 1 do
  begin
    DockPos := PDockPos(FItems[I]);
    if DockPos^.Control = AControl then
    begin
      if DockPos^.Break and (I < FItems.Count - 1) then
        PDockPos(FItems[I + 1])^.Break := True;
      FreeDockPos(FItems, DockPos);
      Break;
    end;
  end;
end;

procedure TagCustomControlBar.GetControlInfo(AControl: TControl; var Insets: TRect;
  var PreferredSize, RowCount: Integer);
begin
  if RowCount = 0 then RowCount := 1;
  if Assigned(FOnBandInfo) then FOnBandInfo(Self, AControl, Insets,
    PreferredSize, RowCount);
end;

procedure TagCustomControlBar.PaintControlFrame(Canvas: TCanvas; AControl: TControl;
  var ARect: TRect);
const
  Offset = 3;
var
  R: TRect;
  Options: TagBandPaintOptions;

  procedure PaintControlFrameV;

    procedure DrawGrabber;
    begin
      with Canvas, R do
      begin
        Pen.Color := clBtnHighlight;
        MoveTo(R.Left, R.Top+2);
        LineTo(R.Left, R.Top);
        LineTo(R.Right+1, R.Top);
        Pen.Color := clBtnShadow;
        MoveTo(R.Left, R.Bottom);
        LineTo(R.Right, R.Bottom);
        LineTo(R.Right, R.Top);
      end;
    end;

  begin
    Options := [bpoGrabber, bpoFrame];
    DoBandPaint(AControl, Canvas, ARect, Options);
    with Canvas do
    begin
      if bpoFrame in Options then
        DrawEdge(Handle, ARect, BDR_RAISEDINNER, BF_RECT);
      if bpoGrabber in Options then
      begin
        R := Rect(ARect.Left + 2, ARect.Top + Offset, ARect.Right - 3,
          ARect.Top + Offset + 2);
        DrawGrabber;
        OffsetRect(R, 0, 3);
        DrawGrabber;
      end;
    end;
  end;

  procedure DrawGrabber;
  begin
    with Canvas, R do
    begin
      Pen.Color := clBtnHighlight;
      MoveTo(R.Left+2, R.Top);
      LineTo(R.Left, R.Top);
      LineTo(R.Left, R.Bottom+1);
      Pen.Color := clBtnShadow;
      MoveTo(R.Right, R.Top);
      LineTo(R.Right, R.Bottom);
      LineTo(R.Left, R.Bottom);
    end;
  end;

begin
  if FVertical then begin
    PaintControlFrameV;
    exit;
  end;

  Options := [bpoGrabber, bpoFrame];
  DoBandPaint(AControl, Canvas, ARect, Options);
  with Canvas do
  begin
    if bpoFrame in Options then
      DrawEdge(Handle, ARect, BDR_RAISEDINNER, BF_RECT);
    if bpoGrabber in Options then
    begin
      R := Rect(ARect.Left + Offset, ARect.Top + 2, ARect.Left + Offset + 2,
        ARect.Bottom - 3);
      DrawGrabber;
      OffsetRect(R, 3, 0);
      DrawGrabber;
    end;
  end;
end;

procedure TagCustomControlBar.Paint;
var
  I, J: Integer;
  DockPos: PDockPos;
  Control: TControl;
  R: TRect;
  Save: Boolean;
begin
  {$IFNDEF D4}
  FDrawBevel:=true;
  inherited Paint;
  FDrawBevel:=false;
  {$ENDIF}
  with Canvas do
  begin
    Brush.Color := Color;
    R := ClientRect;
    if Picture.Graphic <> nil then
    begin
      Save := FDrawing;
      FDrawing := True;
      try
        { Tile image across client area }
        for I := 0 to (R.Right - R.Left) div Picture.Width do
          for J := 0 to (R.Bottom - R.Top) div Picture.Height do
            Draw(I * Picture.Width, J * Picture.Height, Picture.Graphic);
      finally
        FDrawing := Save;
      end
    end
    else
      FillRect(R);
    if Assigned(FOnPaint) then FOnPaint(Self);
    { Draw grabbers and frames for each control }
    for I := 0 to FItems.Count - 1 do
    begin
      DockPos := PDockPos(FItems[I]);
      Control := DockPos^.Control;
      if (DockPos^.Parent = nil) and ((csDesigning in ComponentState) or
        Control.Visible) then
      begin
        R := Control.BoundsRect;
        with DockPos^.Insets do
        begin
          Dec(R.Left, Left);
          Dec(R.Top, Top);
          Inc(R.Right, Right);
          Inc(R.Bottom, Bottom);
        end;
        PaintControlFrame(Canvas, Control, R);
      end;
    end;
  end;
end;

function TagCustomControlBar.HitTest(X, Y: Integer): TControl;
var
  DockPos: PDockPos;
begin
  DockPos := HitTest2(X, Y);
  if DockPos <> nil then
    Result := DockPos^.Control else
    Result := nil;
end;

function TagCustomControlBar.HitTest2(X, Y: Integer): Pointer;
var
  I: Integer;
  R: TRect;
begin
  for I := 0 to FItems.Count - 1 do
  begin
    Result := PDockPos(FItems[I]);
    with PDockPos(Result)^ do
      if (Parent = nil) and ((csDesigning in ComponentState) or
        Control.Visible) then
      begin
        R := Control.BoundsRect;
        with Insets do
        begin
          Dec(R.Left, Left);
          Dec(R.Top, Top);
          Inc(R.Right, Right);
          Inc(R.Bottom, Bottom);
        end;
        if PtInRect(R, Point(X, Y)) then Exit;
      end;
  end;
  Result := nil;
end;

procedure TagCustomControlBar.DoAlignControl(AControl: TControl);
var
  Rect: TRect;
begin
  if not HandleAllocated or (csDestroying in ComponentState) then Exit;
  DisableAlign;
  try
    Rect := GetClientRect;
    AlignControls(AControl, Rect);
  finally
    ControlState := ControlState - [csAlignmentNeeded];
    EnableAlign;
  end;
end;

procedure TagCustomControlBar.CNKeyDown(var Message: TWMKeyDown);
var
  DockPos: PDockPos;
  P: TPoint;
begin
  inherited;
  if (Message.CharCode = VK_CONTROL) and not (csDesigning in ComponentState) and
    AutoDrag and (FDragControl <> nil) then
  begin
    DockPos := FindPos(FDragControl);
    if (DockPos <> nil) and (DockPos^.Control <> nil) then
      with DockPos^ do
      begin
        GetCursorPos(P);
        MapWindowPoints(0, Handle, P, 1);
        DragControl(Control, P.X, P.Y, True);
        Exit;
      end;
  end;
end;

procedure TagCustomControlBar.ResetDockItems;
var
  I: Integer;
begin
  for I := FItems.Count - 1 downto 0 do
    FreeMem(PDockPos(FItems[I]), SizeOf(TDockPos));
  FItems.Clear;
  FDragControl := nil;
  FDockingControl := nil;
  DoAlignControl(nil);
end;

procedure TagCustomControlBar.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DockPos: PDockPos;
begin
  inherited MouseDown(Button, Shift, X, Y);
  if MouseCapture then
  begin
    ResetDockItems;
    if FDragControl <> nil then
      DockPos := FindPos(FDragControl) else
      DockPos := HitTest2(X, Y);
    if (DockPos <> nil) and (not (ssDouble in Shift) or not (AutoDrag or
      (ssDouble in Shift)) or (csDesigning in ComponentState) or
      not DragControl(DockPos^.Control, X, Y, False)) then
    begin
      FDragControl := DockPos^.Control;
      if FDockingControl = FDragControl then
        FDockingControl := nil
      else
        StickControls;
      FDragOffset := Point(DockPos^.TempPos.X - X, DockPos^.TempPos.Y - Y);
    end;
  end;
end;

procedure TagCustomControlBar.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  DockPos: PDockPos;
  Delta: Integer;

  procedure MouseMoveV;
  begin
    inherited MouseMove(Shift, X, Y);
    if MouseCapture then
    begin
      if FDragControl <> nil then
      begin
        DockPos := FindPos(FDragControl);
        if DockPos <> nil then
          with DockPos^ do
          begin
            Pos.X := X + FDragOffset.X;
            Pos.Y := Y + FDragOffset.Y;
            TempPos := Pos;
            TempWidth := Control.Height + Insets.Top + Insets.Bottom;
            { Detect a float operation }
            if not (csDesigning in ComponentState) and AutoDrag then
            begin
              Delta := Control.Width;
              if (Pos.X < -Delta) or (Pos.Y < -Delta) or
                (Pos.X > ClientWidth + Delta) or (Pos.Y > ClientHeight + Delta) then
              begin
                if DragControl(Control, X, Y, True) then Exit;
              end;
            end;
            DoAlignControl(Control);
          end;
      end;
    end;
  end;

begin
  if FVertical then begin
    MouseMoveV;
    exit;
  end;

  inherited MouseMove(Shift, X, Y);
  if MouseCapture then
  begin
    if FDragControl <> nil then
    begin
      DockPos := FindPos(FDragControl);
      if DockPos <> nil then
        with DockPos^ do
        begin
          Pos.X := X + FDragOffset.X;
          Pos.Y := Y + FDragOffset.Y;
          TempPos := Pos;
          TempWidth := Control.Width + Insets.Left + Insets.Right;
          { Detect a float operation }
          if not (csDesigning in ComponentState) and AutoDrag then
          begin
            Delta := Control.Height;
            if (Pos.X < -Delta) or (Pos.Y < -Delta) or
              (Pos.X > ClientWidth + Delta) or (Pos.Y > ClientHeight + Delta) then
            begin
              if DragControl(Control, X, Y, True) then Exit;
            end;
          end;
          DoAlignControl(Control);
        end;
    end;
  end;
end;

procedure TagCustomControlBar.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var
  Control: TControl;
begin
  if FDragControl <> nil then
  begin
    Control := FDragControl;
    FDragControl := nil;
    if FDockingControl = Control then
      FDockingControl := nil
    else
      StickControls;
  end;
  inherited MouseUp(Button, Shift, X, Y);
end;

function TagCustomControlBar.FindPos(AControl: TControl): Pointer;
var
  I: Integer;
begin
  for I := 0 to FItems.Count - 1 do
    with PDockPos(FItems[I])^ do
    begin
      if (Parent = nil) and (Control = AControl) then
      begin
        Result := FItems[I];
        Exit;
      end;
    end;
  Result := nil;
end;

function TagCustomControlBar.DragControl(AControl: TControl; X, Y: Integer;
  KeepCapture: Boolean): Boolean;
begin
  Result := True;
  if (AControl <> nil) and Assigned(FOnBandDrag) then
    FOnBandDrag(Self, AControl, Result);
  if Result then
    AControl.BeginDrag(True);
end;

{$IFDEF D4}
procedure TagCustomControlBar.DockOver(Source: TDragDockObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  inherited DockOver(Source, X, Y, State, Accept);
  if AutoDrag and Accept and (State = dsDragEnter) and Source.Control.Floating then
  begin
    FDragControl := Source.Control;
    FDragControl.EndDrag(True);
    PostMessage(Handle, WM_LBUTTONDOWN, MK_LBUTTON, MakeLong(FDragControl.Left,
      FDragControl.Top));
  end;
end;

procedure TagCustomControlBar.GetSiteInfo(Client: TControl; var InfluenceRect: TRect;
  MousePos: TPoint; var CanDock: Boolean);
begin
  inherited GetSiteInfo(Client, InfluenceRect, MousePos, CanDock);
  CanDock := CanDock and not FFloating;
end;
{$ENDIF}

procedure TagCustomControlBar.DoBandMove(Control: TControl; var ARect: TRect);
begin
  if Assigned(FOnBandMove) then FOnBandMove(Self, Control, ARect);
end;

procedure TagCustomControlBar.DoBandPaint(Control: TControl; Canvas: TCanvas;
  var ARect: TRect; var Options: TagBandPaintOptions);
begin
  if Assigned(FOnBandPaint) then FOnBandPaint(Self, Control, Canvas, ARect, Options);
end;

function TagCustomControlBar.GetPalette: HPALETTE;
begin
  Result := 0;
  if FPicture.Graphic <> nil then
    Result := FPicture.Graphic.Palette;
end;

procedure TagCustomControlBar.SetPicture(const Value: TPicture);
begin
  FPicture.Assign(Value);
end;

function TagCustomControlBar.DoPaletteChange: Boolean;
var
  ParentForm: TCustomForm;
  Tmp: TGraphic;
begin
  Result := False;
  Tmp := Picture.Graphic;
  if Visible and (not (csLoading in ComponentState)) and (Tmp <> nil) and
    (Tmp.PaletteModified) then
  begin
    if (Tmp.Palette = 0) then
      Tmp.PaletteModified := False
    else
    begin
      ParentForm := GetParentForm(Self);
      if Assigned(ParentForm) and ParentForm.Active and Parentform.HandleAllocated then
      begin
        if FDrawing then
          ParentForm.Perform(WM_QUERYNEWPALETTE, 0, 0)
        else
          PostMessage(ParentForm.Handle, WM_QUERYNEWPALETTE, 0, 0);
        Result := True;
        Tmp.PaletteModified := False;
      end;
    end;
  end;
end;

procedure TagCustomControlBar.PictureChanged(Sender: TObject);
begin
  if Picture.Graphic <> nil then
    if DoPaletteChange and FDrawing then Update;
  if not FDrawing then Invalidate;
end;

procedure TagCustomControlBar.CMControlListChange(var Message: TCMControlListChange);
begin
  inherited;
  if not Message.Inserting then
  begin
    if Message.Control = FDragControl then
      FDragControl := nil;
    UnDockControl(Message.Control);
    if AutoSize then AdjustSize;
    Invalidate;
  end;
end;

procedure TagCustomControlBar.CMDesignHitTest(var Message: TCMDesignHitTest);
begin
  Message.Result := Ord((FDragControl <> nil) or
    (HitTest(Message.XPos, Message.YPos) <> nil));
end;

{$IFNDEF D4}
procedure TagCustomControlBar.AdjustSize;
var
  w,h: integer;
begin
  if AutoSize then begin
    w:=Width; h:=Height;
    if CanAutoSize(w,h) then begin
      Width:=w; Height:=h;
    end;
  end;
end;

procedure TagCustomControlBar.SetAutoSize(Value: boolean);
begin
  if Value <> FAutoSize then begin
    FAutoSize:=Value;
    AdjustSize;
  end;
end;

function TagCustomControlBar.GetClientRect: TRect;
var
  BevelSize: Integer;
begin
  Result:=inherited GetClientRect;
  if FDrawBevel then exit;
  BevelSize := BorderWidth;
  if BevelOuter <> bvNone then Inc(BevelSize, BevelWidth);
  if BevelInner <> bvNone then Inc(BevelSize, BevelWidth);
  InflateRect(Result, -BevelSize, -BevelSize);
end;
{$ENDIF}

end.

