{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agControls;

interface

uses Classes, Controls, StdCtrls;

type
  TVertAlignment = (vaTopJustify, vaCenter, vaBottomJustify);
  TStyle = (vsNormal, vsFlat, vsPushButton);

  TagCheckBox = class(TCheckBox)
  private
    FHorzAlign: TAlignment;
    FVertAlign: TVertAlignment;
    FWordWrap: Boolean;
    FStyle: TStyle;
    procedure SetHorzAlign(Value: TAlignment);
    procedure SetVertAlign(Value: TVertAlignment);
    procedure SetWordWrap(Value: boolean);
    procedure SetStyle(Value: TStyle);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property HorizontalAlignment: TAlignment read FHorzAlign write SetHorzAlign;
    property VerticalAlignment: TVertAlignment read FVertAlign write SetVertAlign;
    property WordWrap: boolean read FWordWrap write SetWordWrap;
    property Style: TStyle read FStyle write SetStyle;
  end;

  TagRadioButton = class(TRadioButton)
  private
    FHorzAlign: TAlignment;
    FVertAlign: TVertAlignment;
    FWordWrap: Boolean;
    FStyle: TStyle;
    procedure SetHorzAlign(Value: TAlignment);
    procedure SetVertAlign(Value: TVertAlignment);
    procedure SetWordWrap(Value: boolean);
    procedure SetStyle(Value: TStyle);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property HorizontalAlignment: TAlignment read FHorzAlign write SetHorzAlign;
    property VerticalAlignment: TVertAlignment read FVertAlign write SetVertAlign;
    property WordWrap: boolean read FWordWrap write SetWordWrap;
    property Style: TStyle read FStyle write SetStyle;
  end;

implementation

uses Windows;

procedure CreateParamsInternal(var Params: TCreateParams; AHorzAlign: TAlignment;
  AVertAlign: TVertAlignment; AStyle: TStyle; AWordWrap: boolean);
begin
  Params.Style:=Params.Style and not(BS_LEFT or BS_RIGHT or BS_CENTER or
    BS_TOP or BS_BOTTOM or BS_VCENTER or BS_FLAT or BS_PUSHLIKE or BS_MULTILINE);
  case AHorzAlign of
    taLeftJustify: Params.Style:=Params.Style or BS_LEFT;
    taRightJustify: Params.Style:=Params.Style or BS_RIGHT;
    taCenter: Params.Style:=Params.Style or BS_CENTER;
  end;
  case AVertAlign of
    vaTopJustify: Params.Style:=Params.Style or BS_TOP;
    vaBottomJustify: Params.Style:=Params.Style or BS_BOTTOM;
    vaCenter: Params.Style:=Params.Style or BS_VCENTER;
  end;
  case AStyle of
    vsFlat: Params.Style:=Params.Style or BS_FLAT;
    vsPushButton: Params.Style:=Params.Style or BS_PUSHLIKE;
  end;
  if AWordWrap then Params.Style:=Params.Style or BS_MULTILINE;
end;


{ TagCheckBox }

constructor TagCheckBox.Create(AOwner: TComponent);
begin
  FWordWrap:=true;
  inherited;
end;

procedure TagCheckBox.CreateParams(var Params: TCreateParams);
begin
  inherited;
  CreateParamsInternal(Params,FHorzAlign,FVertAlign,FStyle,FWordWrap);
end;

procedure TagCheckBox.SetHorzAlign(Value: TAlignment);
begin
  if Value <> FHorzAlign then begin
    FHorzAlign:=Value;
    RecreateWnd;
  end;
end;

procedure TagCheckBox.SetStyle(Value: TStyle);
begin
  if Value <> FStyle then begin
    FStyle:=Value;
    RecreateWnd;
  end;
end;

procedure TagCheckBox.SetVertAlign(Value: TVertAlignment);
begin
  if Value <> FVertAlign then begin
    FVertAlign:=Value;
    RecreateWnd;
  end;
end;

procedure TagCheckBox.SetWordWrap(Value: boolean);
begin
  if Value <> FWordWrap then begin
    FWordWrap:=Value;
    RecreateWnd;
  end;
end;


{ TagRadioButton }

constructor TagRadioButton.Create(AOwner: TComponent);
begin
  FWordWrap:=true;
  inherited;
end;

procedure TagRadioButton.CreateParams(var Params: TCreateParams);
begin
  inherited;
  CreateParamsInternal(Params,FHorzAlign,FVertAlign,FStyle,FWordWrap);
end;

procedure TagRadioButton.SetHorzAlign(Value: TAlignment);
begin
  if Value <> FHorzAlign then begin
    FHorzAlign:=Value;
    RecreateWnd;
  end;
end;

procedure TagRadioButton.SetStyle(Value: TStyle);
begin
  if Value <> FStyle then begin
    FStyle:=Value;
    RecreateWnd;
  end;
end;

procedure TagRadioButton.SetVertAlign(Value: TVertAlignment);
begin
  if Value <> FVertAlign then begin
    FVertAlign:=Value;
    RecreateWnd;
  end;
end;

procedure TagRadioButton.SetWordWrap(Value: boolean);
begin
  if Value <> FWordWrap then begin
    FWordWrap:=Value;
    RecreateWnd;
  end;
end;

end.
