{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agDragDrop;

interface

uses Windows, Classes, DropSource, DropTarget;

type
  TagDropFileSource = class(TDropFileSource);
  TagDropTextSource = class(TDropTextSource);

  TTargetFilePasteEvent = procedure(Sender: TObject; Files: TStrings) of object;
  
  TagDropFileTarget = class(TDropFileTarget)
  private
    fPaste: TTargetFilePasteEvent;
    fKeyState: longint;
  protected
    function DoDragOver(grfKeyState: Longint; pt: TPoint;
             var dwEffect: Longint): HRESULT; override;
  public
    function CheckClipboard: boolean;
    procedure PasteFromClipboard;
    property DragOverKeyState: longint read fKeyState;
  published
    property OnPaste: TTargetFilePasteEvent read fPaste write fPaste;
  end;

  TTargetTextPasteEvent = procedure(Sender: TObject; Text: String) of object;

  TagDropTextTarget = class(TDropTextTarget)
  private
    fPaste: TTargetTextPasteEvent;
    fKeyState: longint;
  protected
    function DoDragOver(grfKeyState: Longint; pt: TPoint;
             var dwEffect: Longint): HRESULT; override;
  public
    function CheckClipboard: boolean;
    procedure PasteFromClipboard;
    property DragOverKeyState: longint read fKeyState;
  published
    property OnPaste: TTargetTextPasteEvent read fPaste write fPaste;
  end;

implementation

uses SysUtils, Clipbrd, ShlObj;

{ TagDropFileTarget }

function TagDropFileTarget.CheckClipboard: boolean;
var
  i: integer;
begin
  Result:=false;
  for i:=0 to Clipboard.FormatCount-1 do
    if Clipboard.Formats[i] = CF_HDROP then begin
      Result:=true;
      break;
    end;
end;

procedure TagDropFileTarget.PasteFromClipboard;
var
  DropFiles: PDropFiles;
  Filename: PChar;
  s: string;
  hData: THandle;
  fFiles: TStringList;
begin
  fFiles:=TStringList.Create;
  try
    Clipboard.Open;
    try
      if not(CheckClipboard) then exit;
      hData:=Clipboard.GetAsHandle(CF_HDROP);
      DropFiles:=PDropFiles(GlobalLock(hData));
      try
        Filename:=PChar(DropFiles)+DropFiles^.pFiles;
        while (Filename^ <> #0) do
        begin
          if (DropFiles^.fWide) then // -> NT4 compatability
          begin
            s:=WideCharToString(PWideChar(Filename));
            inc(Filename,(Length(s)+1)*2);
          end else
          begin
            s:=StrPas(Filename);
            inc(Filename,Length(s)+1);
          end;
          fFiles.Add(s);
        end;
      finally
        GlobalUnlock(hData);
      end;
    finally
      Clipboard.Close;
    end;
    if Assigned(fPaste) then fPaste(Self, fFiles);
  finally
    fFiles.Free;
  end;
end;

function TagDropFileTarget.DoDragOver(grfKeyState: Longint; pt: TPoint;
  var dwEffect: Longint): HRESULT;
begin
  Result:=inherited DoDragOver(grfKeyState,pt,dwEffect);
  fKeyState:=grfKeyState;
end;

{ TagDropTextTarget }

function TagDropTextTarget.CheckClipboard: boolean;
var
  i: integer;
begin
  Result:=false;
  for i:=0 to Clipboard.FormatCount-1 do
    if Clipboard.Formats[i] = CF_TEXT then begin
      Result:=true;
      break;
    end;
end;

procedure TagDropTextTarget.PasteFromClipboard;
var
  cText: pchar;
  hData: THandle;
  fText: string;
begin
  Clipboard.Open;
  try
    if not(CheckClipboard) then Exit;
    hData:=Clipboard.GetAsHandle(CF_TEXT);
    cText:=PChar(GlobalLock(hData));
    fText:=cText;
    GlobalUnlock(hData);
  finally
    Clipboard.Close;
  end;
  if Assigned(fPaste) then fPaste(Self, fText);
end;

function TagDropTextTarget.DoDragOver(grfKeyState: Longint; pt: TPoint;
  var dwEffect: Longint): HRESULT;
begin
  Result:=inherited DoDragOver(grfKeyState,pt,dwEffect);
  fKeyState:=grfKeyState;
end;

end.
