{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agMenuBar;

interface

uses Windows, Classes, Controls, ComCtrls, Menus;

type
  TagMenuBar = class(TToolBar)
  private
    FMenu: TMainMenu;
    FIsMain: Boolean;
    procedure SetMenu(const Value: TMainMenu);
  protected
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure Notification( AComponent: TComponent; Operation: TOperation ); override;
  public
    constructor Create(AOwner: TComponent); override;
    class procedure RemovePublished;
  published
    property EdgeBorders default [];
    property Menu: TMainMenu read FMenu write SetMenu;
  end;

implementation

uses Forms, ImgList, DsgnIntf;

constructor TagMenuBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIsMain:=False;
  Flat:=True;
  ShowCaptions:=True;
  EdgeBorders:=[];
  ControlStyle:=[csCaptureMouse, csClickEvents, csDoubleClicks,
    csMenuEvents, csSetCaption];
  Wrapable:=false;
  Align:=alNone;
  AutoSize:=True;
  List:=false;
end;

procedure TagMenuBar.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

procedure TagMenuBar.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
 if (AComponent = FMenu) and (Operation = opRemove) then SetMenu(nil);
 inherited;
end;

class procedure TagMenuBar.RemovePublished;
begin
  RegisterPropertyEditor(TypeInfo(TAlign),TagMenuBar,'Align',nil);
  RegisterPropertyEditor(TypeInfo(boolean),TagMenuBar,'AutoSize',nil);
  RegisterPropertyEditor(TypeInfo(integer),TagMenuBar,'ButtonHeight',nil);
  RegisterPropertyEditor(TypeInfo(integer),TagMenuBar,'ButtonWidth',nil);
  RegisterPropertyEditor(TypeInfo(boolean),TagMenuBar,'Ctl3D',nil);
  RegisterPropertyEditor(TypeInfo(TCustomImageList),TagMenuBar,'DisabledImages',nil);
  RegisterPropertyEditor(TypeInfo(boolean),TagMenuBar,'Flat',nil);
  RegisterPropertyEditor(TypeInfo(integer),TagMenuBar,'Height',nil);
  RegisterPropertyEditor(TypeInfo(TCustomImageList),TagMenuBar,'HotImages',nil);
  RegisterPropertyEditor(TypeInfo(TCustomImageList),TagMenuBar,'Images',nil);
  RegisterPropertyEditor(TypeInfo(boolean),TagMenuBar,'List',nil);
  RegisterPropertyEditor(TypeInfo(boolean),TagMenuBar,'ShowCaptions',nil);
  RegisterPropertyEditor(TypeInfo(boolean),TagMenuBar,'Wrapable',nil);
end;

procedure TagMenuBar.SetMenu(const Value: TMainMenu);
var
  i: Integer;
  Button: TToolButton;
begin
  if FMenu = Value then exit;

  if Assigned(FMenu) then begin
    if FIsMain then (Owner as TForm).Menu:=FMenu;
    FIsMain:=False;
    for i:=ButtonCount - 1 downto 0 do
      Buttons[i].Free;
  end;

  FMenu:=Value;
  if not Assigned(FMenu) then exit;

  if Owner is TForm then
    if (Owner as TForm).Menu = FMenu then begin
      FIsMain:=True;
      (Owner as TForm).Menu:=nil;
    end;

  for i:=0 to FMenu.Items.Count-1 do begin
    Button:=TToolButton.Create(Self);
    try
      Button.Grouped:=True;
      Button.Parent:=Self;
      Button.AutoSize:=True;
    except
      Button.Free;
      raise;
    end;
  end;
  { Copy attributes from each menu item }
  for i:=0 to FMenu.Items.Count-1 do
    Buttons[i].MenuItem:=FMenu.Items[i];
end;

end.
