{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agNAD;

interface

procedure NadPlay;
procedure NadPause;
procedure NadStop;
procedure NadNextTrack;
procedure NadPreviousTrack;
procedure NadExpandWindow;

procedure NadResetPlayList;
procedure NadAddToPlayList(FileName: string);
procedure NadPlayFile(FileName: string);
procedure NadPlayItem(Item: integer);
procedure NadPlayItemName(const ItemName: string);
procedure NadPlayFromStart;
procedure NadSavePlayList(const FileName: string);

function NadGetMaxFrame: longint;
function NadGetCurrentFrame: longint;
procedure NadSetCurrentFrame(Frame: longint);
function NadGetCurrentPlayTime: longint;

function NadGetVolume: longint;
procedure NadSetVolume(Volume: longint);

function NadExecute(const ANadFileName: string): boolean;
procedure NadShow;
procedure NadHide;

type
  TNadState = (stPlaying, stPaused, stStopped);

function NadState: TNadState;

var
  NadFileName: string;

  
implementation

uses SysUtils, Windows, Messages, ShellApi;

const
  ID_PLAY          = 40010;
  ID_STOP          = 40011;
  ID_PAUSE         = 40012;
  ID_NEXTTRACK     = 40013;
  ID_PREVIOUSTRACK = 40014;
  ID_EXPANDWINDOW  = 40105; // this causes the NAD window to 'expand'
                            // or contract (like pressing the notch)

  //The NAD Communication System
  NCM_RESETPLAYLIST = 0;
  NCM_ADDTOPLAYLIST = 1;
  NCM_PLAYFILE      = 2;
  NCM_PLAYITEM      = 3;
  NCM_PLAYFROMSTART = 4;
  NCM_SAVEPLAYLIST  = 5;
  NCM_SHUFFLE       = 6;

  WM_GETCURRENTPLAYTIME = WM_APP + $700; // returns playtime in MS
  WM_GETCURRENTFRAME    = WM_APP + $701; // returns the last decoded frame
  WM_GETMAXFRAME        = WM_APP + $702; // return the ESTIMATED number of frames
                                         // this is -1 for a URL stream.
  WM_GETVOLUME          = WM_APP + $703; // returns the CURRENT volume Level
  WM_SETVOLUME          = WM_APP + $704; // lParam = the NEW volume
  // VOLUME values are between 0 and 64, setting to more (or less) will have NO effect...

  WM_SETCURRENTFRAME    = WM_APP + $705; // lParam = the Frame to jump to

  WM_GETPLAYSTATUS      = WM_APP + $706; // returns integer value(see below)

  // these are the return value from WM_SETPLAYSTATUS
  STAT_PLAYING = 0;
  STAT_PAUSED  = 1;
  STAT_STOPPED = 2;

var
  cds: TCopyDataStruct;
  NadWindow: HWnd;

function GetNadWindow: HWnd;
begin
  NadWindow:=FindWindow('NadMainWindow',nil);
  if NadWindow = 0 then begin
    if ShellExecute(0,'open',PChar(NadFileName),'',PChar(ExtractFilePath(NadFileName)),
      SW_HIDE) > 32 then
    begin
      Sleep(200);
      repeat
        NadWindow:=FindWindow('NadMainWindow',nil);
      until NadWindow <> 0;
    end;
  end;
  Result:=NadWindow;
end;

function NadExecute(const ANadFileName: string): boolean;
begin
  NadFileName:=ANadFileName;
  Result:=(GetNadWindow <> 0);
end;

procedure NadPlayFile(FileName: string);
begin
  cds.dwData:=NCM_PLAYFILE;
  cds.cbData:=length(FileName);
  cds.lpData:=PChar(FileName);
  SendMessage(GetNadWindow,WM_COPYDATA,0,integer(@cds));
end;

procedure NadAddToPlayList(FileName: string);
begin
  cds.dwData:=NCM_ADDTOPLAYLIST;
  cds.cbData:=length(FileName);
  cds.lpData:=PChar(FileName);
  SendMessage(GetNadWindow,WM_COPYDATA,0,integer(@cds));
end;

procedure NadPlay;
begin
  SendMessage(GetNadWindow,WM_COMMAND,MAKEWPARAM(ID_PLAY,0),0);
end;

procedure NadStop;
begin
  SendMessage(GetNadWindow,WM_COMMAND,MAKEWPARAM(ID_STOP,0),0);
end;

function NadGetMaxFrame: longint;
begin
  Result:=SendMessage(GetNadWindow,WM_GETMAXFRAME,0,0);
end;

function NadGetCurrentFrame: longint;
begin
  Result:=SendMessage(GetNadWindow,WM_GETCURRENTFRAME,0,0);
end;

function NadState: TNadState;
begin
  case SendMessage(GetNadWindow,WM_GETPLAYSTATUS,0,0) of
    STAT_PLAYING: Result:=stPlaying;
    STAT_PAUSED: Result:=stPaused;
    STAT_STOPPED: Result:=stStopped;
    else Result:=stStopped;
  end;
end;

procedure NadResetPlayList;
begin
  cds.dwData:=NCM_RESETPLAYLIST;
  cds.cbData:=0;
  cds.lpData:=nil;
  SendMessage(GetNadWindow,WM_COPYDATA,0,integer(@cds));
end;

procedure NadSetCurrentFrame(Frame: longint);
begin
  SendMessage(GetNadWindow,WM_SETCURRENTFRAME,0,Frame);
end;

function NadGetCurrentPlayTime: longint;
begin
  Result:=SendMessage(GetNadWindow,WM_GETCURRENTPLAYTIME,0,0);
end;

procedure NadPause;
begin
  SendMessage(GetNadWindow,WM_COMMAND,MAKEWPARAM(ID_PAUSE,0),0);
end;

function NadGetVolume: longint;
begin
  Result:=SendMessage(GetNadWindow,WM_GETVOLUME,0,0);
end;

procedure NadSetVolume(Volume: longint);
begin
  SendMessage(GetNadWindow,WM_SETVOLUME,0,Volume);
end;

procedure NadShow;
begin
  ShowWindow(GetNadWindow,SW_SHOWNORMAL);
end;

procedure NadHide;
begin
  ShowWindow(GetNadWindow,SW_HIDE);
end;

procedure NadPlayFromStart;
begin
  cds.dwData:=NCM_PLAYFROMSTART;
  cds.cbData:=0;
  cds.lpData:=nil;
  SendMessage(GetNadWindow,WM_COPYDATA,0,integer(@cds));
end;

procedure NadNextTrack;
begin
  SendMessage(GetNadWindow,WM_COMMAND,MAKEWPARAM(ID_NEXTTRACK,0),0);
end;

procedure NadPreviousTrack;
begin
  SendMessage(GetNadWindow,WM_COMMAND,MAKEWPARAM(ID_PREVIOUSTRACK,0),0);
end;

procedure NadExpandWindow;
begin
  SendMessage(GetNadWindow,WM_COMMAND,MAKEWPARAM(ID_EXPANDWINDOW,0),0);
end;

procedure NadPlayItem(Item: integer);
begin
  cds.dwData:=NCM_PLAYITEM;
  cds.cbData:=Item;
  cds.lpData:=nil;
  SendMessage(GetNadWindow,WM_COPYDATA,0,integer(@cds));
end;

procedure NadPlayItemName(const ItemName: string);
begin
  cds.dwData:=NCM_PLAYITEM;
  cds.cbData:=-1;
  cds.lpData:=PChar(ItemName+#0#0);
  SendMessage(GetNadWindow,WM_COPYDATA,0,integer(@cds));
end;

procedure NadSavePlayList(const FileName: string);
begin
  cds.dwData:=NCM_SAVEPLAYLIST;
  cds.cbData:=length(FileName);
  cds.lpData:=PChar(FileName);
  SendMessage(GetNadWindow,WM_COPYDATA,0,integer(@cds));
end;

initialization
  NadWindow:=0;

finalization
  if NadWindow <> 0 then SendMessage(NadWindow,WM_CLOSE,0,0);

end.
