{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agProgress;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ExtCtrls;

type
  TDrawStyle = (dsNormal, dsInvert);
  TTextAlign = (taCenter, taLeft, taRight, taFlat);
  TBorderStyle = (bsNone, bsShallow, bsDeep);
  TGetTextEvent = procedure(Position,Percent: integer; var Text: string) of object;

  TagProgress = class(TCustomControl)
  private
    FPerc:          integer;
    FMin:           integer;
    FMax:           integer;
    FPos:           integer;
    FPosition:      integer;
    FPColor:        TColor;
    FText:          string;
    FTextAlign:     TTextAlign;
    FDrawStyle:     TDrawStyle;
    FCtl3D:         boolean;
    FIndent:        integer;
    FShowPos:       boolean;
    FShowPer:       boolean;
    FBorderStyle:   TBorderStyle;
    flgShowAll:     boolean;
    FOnGetText:     TGetTextEvent;
    procedure SetMin(Value: integer);
    procedure SetMax(Value: integer);
    procedure SetPosition(Value: integer);
    procedure SetPColor(Value: TColor);
    procedure SetText(Value: string);
    procedure SetTextAlign(Value: TTextAlign);
    procedure SetDrawStyle(Value: TDrawStyle);
    procedure SetCtl3D(Value: boolean);
    procedure SetIndent(Value: integer);
    procedure SetShowPos(Value: boolean);
    procedure SetShowPer(Value: boolean);
    procedure SetBorderStyle(Value: TBorderStyle);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    property Percent: integer read FPerc;
  published
    property Min: integer read FMin write SetMin;
    property Max: integer read FMax write SetMax;
    property Position: integer read FPosition write SetPosition;
    property ProgressColor: TColor read FPColor write SetPColor;
    property Text: string read FText write SetText;
    property ParentColor default false;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property PopupMenu;
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle default bsShallow;
    property Color default clBtnFace;
    property Font;
    property Align;
    property TextAlign: TTextAlign read FTextAlign write SetTextAlign;
    property DrawStyle: TDrawStyle read FDrawStyle write SetDrawStyle;
    property Ctl3D: boolean read FCtl3D write SetCtl3D;
    property Indent: integer read FIndent write SetIndent;
    property ShowPosition: boolean read FShowPos write SetShowPos;
    property ShowPercent: boolean read FShowPer write SetShowPer;
    property OnGetText: TGetTextEvent read FOnGetText write FOnGetText;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnEnter;
    property OnExit;
    {$IFDEF D4}
    property Anchors;
    property Constraints;
    property OnResize;
    {$ENDIF}
  end;

implementation

constructor TagProgress.Create(AOwner: TComponent);
begin
  inherited;
  Width:=150;
  Height:=19;
  ParentColor:=false;
  FPerc:=0;
  FMin:=0;
  FMax:=100;
  FPos:=0;
  FPosition:=0;
  FPColor:=clHighlight;
  FTextAlign:=taCenter;
  FDrawStyle:=dsInvert;
  FCtl3D:=true;
  FIndent:=0;
  FShowPos:=true;
  FShowPer:=true;
  FBorderStyle:=bsShallow;
  Color:=clBtnFace;
  flgShowAll:=true;
  FOnGetText:=nil;
end;

procedure TagProgress.SetText(Value: string);
begin
  if Value <> FText then begin
    FText:=Value;
    Paint;
  end;
end;

procedure TagProgress.SetMax(Value: integer);
begin
  if Value <> FMax then begin
    if Value <= FMin then Value:=FMin+1;
    FMax:=Value;
    if FPosition > FMax then FPosition:=FMax;
    flgShowAll:=false;
    Paint;
  end;
end;

procedure TagProgress.SetPosition(Value: integer);
begin
  if Value <> FPosition then begin
    if Value < FMin then Value:=FMin;
    if Value > FMax then Value:=FMax;
    FPosition:=Value;
    flgShowAll:=false;
    Paint;
  end;
end;

procedure TagProgress.SetPColor(Value: TColor);
begin
  if Value <> FPColor then begin
    FPColor:=Value;
    Paint;
  end;
end;

procedure TagProgress.Paint;
var
  w,h,deep,col: integer;
  i,x,y: integer;
  lBMP, bmpText: TBitmap;
  s,s1: string;
  iFPerc,iFPos: integer;
begin
  case FBorderStyle of
    bsNone: deep:=0;
    bsShallow: deep:=1;
    bsDeep: if FCtl3D then deep:=2 else deep:=1;
  end;
  iFPerc:=Round(((FPosition-FMin) / (FMax-FMin)) * 100);
  iFPos:=Round(((FPosition-FMin) / (FMax-FMin)) * (Width - FIndent*2 - deep*2));
  if flgShowAll then begin
    FPerc:=iFPerc; FPos:=iFPos;
  end else begin
    flgShowAll:=true;
    if (iFPerc <> FPerc) or (iFPos <> FPos) then begin
      FPerc:=iFPerc; FPos:=iFPos;
    end else
      exit;
  end;
  w:=Width;
  h:=Height;
  lBMP:=TBitmap.Create;
  try
    lBMP.Width:=w;
    lBMP.Height:=h;
    lBMP.Canvas.Font.Assign(Font);
    with lBMP.Canvas do begin
      if Assigned(FOnGetText) then
        FOnGetText(FPosition,FPerc,s)
      else begin
        s1:='';
        if FShowPos then begin
          if FShowPer then s1:=Format('%d%%',[FPerc]) else s1:=Format('%d',[FPosition]);
        end;
        if FText <> '' then
          s:=Format('%s   %s',[FText,s1])
        else
          s:=s1;
      end;
      i:=TextWidth(s);
      case FTextAlign of
        taLeft: x:=4;
        taRight: x:=w-i-4;
        taCenter: x:=(w-i) div 2;
        taFlat:
          begin
            x:=(FPos-i) div 2;
            if x < 4 then x:=4;
            x:=x+FIndent;
          end;
      end;
      y:=(h-TextHeight(s)) div 2;
      if FDrawStyle = dsInvert then begin
        //  ProgressBar
        bmpText:=TBitmap.Create;
        try
          bmpText.Width:=lBMP.Width;
          bmpText.Height:=lBMP.Height;
          bmpText.Canvas.Font.Assign(Font);
          i:=FIndent+deep;
          col:=clBlack;
          with bmpText.Canvas do begin
            Brush.Style:=bsSolid;
            Brush.Color:=Color;
            FillRect(rect(0,0,w,h));
            Brush.Color:=FPColor;
            if FPos > 0 then FillRect(rect(i,i,i+FPos,h-i));
            Brush.Style:=bsClear;
            Font.Color:=col;
            TextOut(x,y,s);
          end;
          Brush.Color:=Color;
          BrushCopy(rect(0,0,i+FPos,h),bmpText,rect(0,0,i+FPos,h),col);
          Brush.Color:=Font.Color;
          BrushCopy(rect(i+FPos,0,w,h),bmpText,rect(i+FPos,0,w,h),col);
        finally
          bmpText.Free;
        end;
      end else begin
        //  ProgressBar
        Brush.Style:=bsSolid;
        Brush.Color:=Color;
        FillRect(rect(0,0,w,h));
        Brush.Color:=FPColor;
        i:=FIndent+deep;
        if FPos > 0 then FillRect(rect(i,i,i+FPos,h-i));
        Brush.Style:=bsClear;
        TextOut(x,y,s);
      end;
      // 
      if FBorderStyle <> bsNone then begin
        Pen.Style:=psSolid;
        if FCtl3D then begin
          Pen.Color:=clBtnShadow;
          MoveTo(0,h-1);
          LineTo(0,0);
          LineTo(w-1,0);
          Pen.Color:=clBtnHighlight;
          LineTo(w-1,h-1);
          LineTo(0,h-1);
          if FBorderStyle = bsDeep then begin
            Pen.Color:=cl3DDkShadow;
            MoveTo(1,h-2);
            LineTo(1,1);
            LineTo(w-2,1);
            Pen.Color:=clBtnFace;
            LineTo(w-2,h-2);
            LineTo(1,h-2);
          end;
        end else begin
          Pen.Color:=clBlack;
          Brush.Style:=bsClear;
          Rectangle(0,0,w,h);
        end;
      end;
    end;
    //  
    Canvas.Copyrect(rect(0,0,w+1,h+1),lBMP.Canvas,rect(0,0,w+1,h+1));
  finally
    lBMP.Free;
  end;
end;

procedure TagProgress.SetTextAlign(Value: TTextAlign);
begin
  if Value <> FTextAlign then begin
    FTextAlign:=Value;
    Paint;
  end;
end;

procedure TagProgress.SetDrawStyle(Value: TDrawStyle);
begin
  if Value <> FDrawStyle then begin
    FDrawStyle:=Value;
    Paint;
  end;
end;

procedure TagProgress.SetCtl3D(Value: boolean);
begin
  if Value <> FCtl3D then begin
    FCtl3D:=Value;
    Paint;
  end;
end;

procedure TagProgress.SetIndent(Value: integer);
begin
  if Value <> FIndent then begin
    FIndent:=Value;
    Paint;
  end;
end;

procedure TagProgress.SetShowPos(Value: boolean);
begin
  if Value <> FShowPos then begin
    FShowPos:=Value;
    Paint;
  end;
end;

procedure TagProgress.SetShowPer(Value: boolean);
begin
  if Value <> FShowPer then begin
    FShowPer:=Value;
    Paint;
  end;
end;

procedure TagProgress.SetMin(Value: integer);
begin
  if Value <> FMin then begin
    if Value >= FMax then Value:=FMax-1;
    FMin:=Value;
    if FPosition < FMin then FPosition:=FMin;
    flgShowAll:=false;
    Paint;
  end;
end;

procedure TagProgress.SetBorderStyle(Value: TBorderStyle);
begin
  if Value <> FBorderStyle then begin
    FBorderStyle:=Value;
    Paint;
  end;
end;

end.
