{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agShell;

interface

uses Windows;

const
  // Root consts
  CSIDL_DESKTOP          = 0;
  CSIDL_PROGRAMS         = 2;
  CSIDL_CONTROLS         = 3;
  CSIDL_PRINTERS         = 4;
  CSIDL_PERSONAL         = 5;
  CSIDL_FAVORITES        = 6;
  CSIDL_STARTUP          = 7;
  CSIDL_RECENT           = 8;
  CSIDL_SENDTO           = 9;
  CSIDL_BITBUCKET        = $a;
  CSIDL_STARTMENU        = $b;
  CSIDL_DESKTOPDIRECTORY = $10;
  CSIDL_DRIVES           = $11;
  CSIDL_NETWORK          = $12;
  CSIDL_NETHOOD          = $13;
  CSIDL_FONTS		 = $14;
  CSIDL_TEMPLATES        = $15;

  // Browsing flags
  BIF_RETURNONLYFSDIRS   = $0001; // For finding a folder to start document searching
  BIF_DONTGOBELOWDOMAIN  = $0002; // For starting the Find Computer
  BIF_STATUSTEXT         = $0004;
  BIF_RETURNFSANCESTORS  = $0008;
  BIF_BROWSEFORCOMPUTER  = $1000; // Browsing for Computers
  BIF_BROWSEFORPRINTER   = $2000; // Browsing for Printers
  BIF_BROWSEINCLUDEFILES = $4000; // Browsing for Everything

function BrowseFor(AOwner: HWnd; DialogText: string; Root,Flags: integer;
  var Res: string): boolean;
  
function BrowseForFolder(AOwner: HWnd; DialogText: string; var Folder: string): boolean;
function BrowseForNetFolder(AOwner: HWnd; DialogText: string; var Folder: string): boolean;
function BrowseForComputer(AOwner: HWnd; DialogText: string; var Comp: string): boolean;
function BrowseForPrinter(AOwner: HWnd; DialogText: string; var Printer: string): boolean;

function ShellExecuteWait(Handle: HWnd; const FileName, Params, StartDir: string;
  ShowCmd: integer): Integer;


type
  TSpecialFolder = (sfDesktop, sfFonts, sfNetHood, sfPersonal, sfPrograms, sfRecent,
    sfSendTo, sfStartMenu, sfStartup, sfTemplates);

function GetSpecialFolderLocation(SpecFolder: TSpecialFolder): string;


implementation

uses ActiveX, ComObj, ShellApi, ShlObj, SysUtils, Messages, agFileUtils;

var
  SelItem: string;

function BrowseHook(Wnd: HWnd; Msg: UINT; LParam: LPARAM; Data: LPARAM): Integer; stdcall;
begin
  Result:=0;
  if Msg = BFFM_INITIALIZED then
    SendMessage(Wnd,BFFM_SETSELECTION,1,Longint(PChar(SelItem)));
end;

function BrowseFor(AOwner: HWnd; DialogText: string; Root,Flags: integer;
  var Res: string): boolean;
var
  BrowseInfo: TBrowseInfo;
  DisplayName,s: string;
  ItemIDList: PItemIDList;
begin
  Result:=false;
  OleCheck(SHGetSpecialFolderLocation(0,Root,BrowseInfo.pidlRoot));
  try
    SetLength(DisplayName,MAX_PATH);
    with BrowseInfo do begin
      hWndOwner:=AOwner;
      pszDisplayName:=PChar(DisplayName);
      lpszTitle:=PChar(DialogText);
      ulFlags:=Flags;
      lpfn:=BrowseHook;
      lParam:=0;
      iImage:=0;
    end;
    SelItem:=Res;
    ItemIDList:=SHBrowseForFolder(BrowseInfo);
    if ItemIDList <> nil then
    try
      if (Flags and BIF_BROWSEFORCOMPUTER) = 0 then begin
        SetLength(s,MAX_PATH);
        Win32Check(SHGetPathFromIDList(ItemIDList,PChar(s)));
      end else
        s:=BrowseInfo.pszDisplayName;
      Res:=string(PChar(s));
      Result:=true;
    finally
      CoTaskMemFree(ItemIDList);
    end;
  finally
    if BrowseInfo.pidlRoot <> nil then CoTaskMemFree(BrowseInfo.pidlRoot);
  end;
end;

function BrowseForFolder(AOwner: HWnd; DialogText: string; var Folder: string): boolean;
begin
  Result:=BrowseFor(AOwner,DialogText,CSIDL_DRIVES,BIF_RETURNONLYFSDIRS or
    BIF_RETURNFSANCESTORS,Folder);
  if Result then Folder:=RemoveBackSlash(Folder);
end;

function BrowseForNetFolder(AOwner: HWnd; DialogText: string; var Folder: string): boolean;
begin
  Result:=BrowseFor(AOwner,DialogText,CSIDL_NETWORK,BIF_RETURNONLYFSDIRS or
    BIF_RETURNFSANCESTORS,Folder);
  if Result then Folder:=RemoveBackSlash(Folder);
end;

function BrowseForComputer(AOwner: HWnd; DialogText: string; var Comp: string): boolean;
begin
  Result:=BrowseFor(AOwner,DialogText,CSIDL_NETWORK,BIF_BROWSEFORCOMPUTER,Comp);
end;

function BrowseForPrinter(AOwner: HWnd; DialogText: string; var Printer: string): boolean;
begin
  Result:=BrowseFor(AOwner,DialogText,CSIDL_NETWORK,BIF_BROWSEFORPRINTER,Printer);
end;

function ShellExecuteWait(Handle: HWnd; const FileName, Params, StartDir: string;
  ShowCmd: integer): Integer;
var
  Info: TShellExecuteInfo;
  ExitCode: DWORD;
  Msg: TMsg;
begin
  Result := -1;
  FillChar(Info, SizeOf(Info), 0);
  Info.cbSize := SizeOf(TShellExecuteInfo);
  with Info do begin
    fMask := SEE_MASK_NOCLOSEPROCESS;
    Wnd := Handle;
    lpFile := PChar(FileName);
    lpParameters := PChar(Params);
    lpDirectory := PChar(StartDir);
    nShow := ShowCmd;
  end;
  if ShellExecuteEx(@Info) then begin
    repeat
      // process messages
      while PeekMessage(Msg, 0, 0, 0, PM_REMOVE) do begin
        if Msg.Message = WM_QUIT then exit;
        TranslateMessage(Msg);
        DispatchMessage(Msg);
      end;
      Sleep(1);
      GetExitCodeProcess(Info.hProcess, ExitCode);
    until ExitCode <> STILL_ACTIVE;
    Result := ExitCode;
  end;
end;


function GetSpecialFolderLocation(SpecFolder: TSpecialFolder): string;
const
  SpecFolderID: array [TSpecialFolder] of integer = (
     CSIDL_DESKTOPDIRECTORY,
     CSIDL_FONTS,
     CSIDL_NETHOOD,
     CSIDL_PERSONAL,
     CSIDL_PROGRAMS,
     CSIDL_RECENT,
     CSIDL_SENDTO,
     CSIDL_STARTMENU,
     CSIDL_STARTUP,
     CSIDL_TEMPLATES
    );
var
  pi: PItemIDList;
begin
  Result:='';
  if SHGetSpecialFolderLocation(0,SpecFolderID[SpecFolder],pi) = NOERROR then begin
    SetLength(Result,MAX_PATH);
    if SHGetPathFromIDList(pi,PChar(Result)) then
      SetLength(Result,StrLen(PChar(Result)))
    else
      SetLength(Result,0);
  end;
end;

end.
