{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agShellLink;

interface

type
  TShowCmd = (scNormal, scMinimized, scMaximized);

  TagShellLink = class(TObject)
  private
    FLnkFile,FCmdLine,FArg,FWorkDir,FDescr,FIconFile: string;
    FShowCmd: TShowCmd;
    FHotKey: word;
    FIconIndex: integer;
  public
    property FileName: string read FLnkFile write FLnkFile;
    property CmdLine: string read FCmdLine write FCmdLine;
    property Arguments: string read FArg write FArg;
    property WorkDir: string read FWorkDir write FWorkDir;
    property ShowCmd: TShowCmd read FShowCmd write FShowCmd;
    property HotKey: word read FHotKey write FHotKey;
    property Description: string read FDescr write FDescr;
    property IconFile: string read FIconFile write FIconFile;
    property IconIndex: integer read FIconIndex write FIconIndex;
    function Load: boolean;
    function LoadFromFile(FileName: string): boolean;
    function Save: boolean;
    function SaveToFile(FileName: string): boolean;
  end;

function CreateShellLink(const LinkFile, CmdLine, Args, WorkDir: string;
  ShowCmd: TShowCmd): boolean;

implementation

uses Windows, SysUtils, ActiveX, ShlObj, ComObj;

function TagShellLink.Load: boolean;
var
  Obj: IUnknown;
  wfn: WideString;
  fd: TWin32FindData;
  i: integer;
begin
  Result:=false;
  if not FileExists(FLnkFile) then exit;
  Obj:=CreateComObject(CLSID_ShellLink);
  wfn:=FLnkFile;
  if (Obj as IPersistFile).Load(PWideChar(wfn),STGM_READ) <> S_OK then exit;
  with Obj as IShellLink do begin
    // get command line
    SetLength(FCmdLine,MAX_PATH);
    if GetPath(PChar(FCmdLine),length(FCmdLine),fd,0) <> NOERROR then exit;
    FCmdLine:=string(PChar(FCmdLine));
    // get arguments
    SetLength(FArg,MAX_PATH);
    if GetArguments(PChar(FArg),length(FArg)) <> NOERROR then exit;
    FArg:=string(PChar(FArg));
    // get working directory
    SetLength(FWorkDir,MAX_PATH);
    if GetWorkingDirectory(PChar(FWorkDir),length(FWorkDir)) <> NOERROR then exit;
    FWorkDir:=string(PChar(FWorkDir));
    // get show command
    if GetShowCmd(i) <> NOERROR then exit;
    case i of
      SW_SHOWMAXIMIZED: FShowCmd:=scMaximized;
      SW_SHOWMINNOACTIVE: FShowCmd:=scMinimized;
      else FShowCmd:=scNormal;
    end;
    // get hot key
    if GetHotKey(FHotKey) <> NOERROR then exit;
    // get description
    SetLength(FDescr,MAX_PATH);
    if GetDescription(PChar(FDescr),length(FDescr)) <> NOERROR then exit;
    FDescr:=string(PChar(FDescr));
    // get icon location
    SetLength(FIconFile,MAX_PATH);
    if GetIconLocation(PChar(FIconFile),length(FIconFile),FIconIndex) <> NOERROR then exit;
    FIconFile:=string(PChar(FIconFile));
  end;
  Result:=true;
end;

function TagShellLink.Save: boolean;
var
  Obj: IUnknown;
  wfn: WideString;
  i: integer;
begin
  Result:=false;
  Obj:=CreateComObject(CLSID_ShellLink);
  with Obj as IShellLink do begin
    // set command line
    if SetPath(PChar(FCmdLine)) <> NOERROR then exit;
    // set arguments
    if SetArguments(PChar(FArg)) <> NOERROR then exit;
    // set working directory
    if SetWorkingDirectory(PChar(FWorkDir)) <> NOERROR then exit;
    // set show command
    case FShowCmd of
      scMaximized: i:=SW_SHOWMAXIMIZED;
      scMinimized: i:=SW_SHOWMINNOACTIVE;
      else i:=SW_SHOWNORMAL;
    end;
    if SetShowCmd(i) <> NOERROR then exit;
    // set hot key
    if SetHotKey(FHotKey) <> NOERROR then exit;
    // set description
    if SetDescription(PChar(FDescr)) <> NOERROR then exit;
    // set icon location
    if SetIconLocation(PChar(FIconFile),FIconIndex) <> NOERROR then exit;
  end;
  wfn:=FLnkFile;
  if (Obj as IPersistFile).Save(PWideChar(wfn),false) <> S_OK then exit;
  Result:=true;
end;

function TagShellLink.LoadFromFile(FileName: string): boolean;
begin
  FLnkFile:=FileName;
  Result:=Load;
end;

function TagShellLink.SaveToFile(FileName: string): boolean;
begin
  FLnkFile:=FileName;
  Result:=Save;
end;

function CreateShellLink(const LinkFile, CmdLine, Args, WorkDir: string;
  ShowCmd: TShowCmd): boolean;
var
  Lnk: TagShellLink;
begin
  Lnk:=TagShellLink.Create;
  try
    Lnk.CmdLine:=CmdLine;
    Lnk.Arguments:=Args;
    Lnk.WorkDir:=WorkDir;
    Lnk.ShowCmd:=ShowCmd;
    Result:=Lnk.SaveToFile(LinkFile);
  finally
    Lnk.Free;
  end;
end;

initialization
  CoInitialize(nil);
finalization
  CoUninitialize;
end.
