{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agStatus;

interface

uses Windows, Classes, Controls, ExtCtrls, ComCtrls;

type
  TagStatusPanel = class(TComponent)
  private
    FStatusBar:          TStatusBar;
    FControl:            TControl;
    FControlParent:      TWinControl;
    FControlBoundsRect:  TRect;
    FOldOnDrawPanel:     TDrawPanelEvent;
    FPanelIndex:         integer;
    FPanelStyle:         TStatusPanelStyle;
    FControlVisible:     boolean;
    procedure SetStatusBar(Value: TStatusBar);
    procedure SetControl(Value: TControl);
    procedure FOnDrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel; const Rect: TRect);
    procedure SetPanelIndex(Value: integer);
    procedure SetPanelStyle;
    procedure PushControl;
    procedure PopControl;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property StatusBar: TStatusBar read FStatusBar write SetStatusBar;
    property Control: TControl read FControl write SetControl;
    property PanelIndex: integer read FPanelIndex write SetPanelIndex;
  end;

implementation

uses Forms;

{ TagStatusPanel }

constructor TagStatusPanel.Create(AOwner: TComponent);
begin
  inherited;
  FPanelIndex:=0;
end;

destructor TagStatusPanel.Destroy;
begin
  if FControl <> nil then begin
    FControl.Parent:=FControlParent;
    FControl.BoundsRect:=FControlBoundsRect;
  end;
  if FStatusBar <> nil then FStatusBar.OnDrawPanel:=FOldOnDrawPanel;
  inherited;
end;

procedure TagStatusPanel.SetControl(Value: TControl);
var
  i: integer;
begin
  if Value <> FControl then begin
    if (Value is TForm) or (Value is TStatusBar) then FControl:=nil
    else begin
      PopControl;
      if Value <> nil then
        for i:=0 to Owner.ComponentCount-1 do
          if Owner.Components[i] is TagStatusPanel then
            if (Owner.Components[i] as TagStatusPanel).Control = Value then
              (Owner.Components[i] as TagStatusPanel).Control:=nil;
      FControl:=Value;
      PushControl;
      if FStatusBar <> nil then FStatusBar.Refresh;
    end;
  end;
end;

procedure TagStatusPanel.SetStatusBar(Value: TStatusBar);
begin
  if Value <> FStatusBar then begin
    if FStatusBar <> nil then begin
      FStatusBar.OnDrawPanel:=FOldOnDrawPanel;
      PopControl;
    end;
    FStatusBar:=Value;
    if FStatusBar <> nil then begin
      FOldOnDrawPanel:=FStatusBar.OnDrawPanel;
      FStatusBar.OnDrawPanel:=FOnDrawPanel;
      FStatusBar.FreeNotification(Self);
      PushControl;
      FStatusBar.Refresh;
    end;
  end;
end;

procedure TagStatusPanel.FOnDrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel;
  const Rect: TRect);
begin
  if (FControl <> nil) and (FPanelIndex = Panel.Index) then begin
    FControl.Top:=Rect.Top;
    FControl.Height:=Rect.Bottom-Rect.Top;
    FControl.Left:=Rect.Left;
    if (Panel.Index = StatusBar.Panels.Count-1) and StatusBar.SizeGrip then
      FControl.Width:=Rect.Right-Rect.Left-13
    else
      FControl.Width:=Rect.Right-Rect.Left;
  end else
    if Assigned(FOldOnDrawPanel) then FOldOnDrawPanel(StatusBar,Panel,Rect);
end;

procedure TagStatusPanel.Notification(AComponent: TComponent; AOperation: TOperation);
begin
  inherited Notification(AComponent, AOperation);
  if AOperation = opRemove then begin
    if AComponent = FControl
      then Control:=nil
      else if AComponent = FStatusBar
        then StatusBar:=nil;
  end;
end;

procedure TagStatusPanel.SetPanelIndex(Value: integer);
begin
  if Value < 0 then Value:=0;
  if Value <> FPanelIndex then begin
    FPanelIndex:=Value;
    SetPanelStyle;
    if FStatusBar <> nil then FStatusBar.Refresh;
  end;
end;

procedure TagStatusPanel.SetPanelStyle;
begin
  if (FStatusBar <> nil) and (FControl <> nil) then
    if FPanelIndex < FStatusBar.Panels.Count then begin
      FPanelStyle:=FStatusBar.Panels[FPanelIndex].Style;
      FStatusBar.Panels[FPanelIndex].Style:=psOwnerDraw;
    end;
end;

procedure TagStatusPanel.PushControl;
begin
  if FControl <> nil then begin
    FControlVisible:=FControl.Visible;
    FControl.FreeNotification(Self);
    FControlParent:=FControl.Parent;
    FControlBoundsRect:=FControl.BoundsRect;
    if FStatusBar <> nil then FControl.Parent:=FStatusBar;
    SetPanelStyle;
    FControl.Visible:=true;
  end;
end;

procedure TagStatusPanel.PopControl;
begin
  if FControl <> nil then begin
    FControl.Visible:=FControlVisible;
    if not(csDestroying in FControl.ComponentState) then begin
      FControl.Parent:=FControlParent;
      FControl.BoundsRect:=FControlBoundsRect;
    end;
  end;
end;

end.

