{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agSvcApp;

interface

uses SvcMgr;

procedure InstallServices(SvcApp: TServiceApplication; Quiet: Boolean);
procedure UninstallServices(SvcApp: TServiceApplication; Quiet: Boolean);
procedure ServiceApplicationRun(SvcApp: TServiceApplication);

implementation

uses SysUtils, Windows, WinSvc, Consts, Dialogs, agStrUtils;

procedure RegisterServices(SvcApp: TServiceApplication; Install,Quiet: Boolean);

  procedure InstallService(Service: TService; SvcMgr: Integer);

    function GetNTDependencies: String;
    var
      i, Len: Integer;
      P: PChar;
    begin
      with Service do begin
        Result := '';
        Len := 0;
        for i := 0 to Dependencies.Count - 1 do
        begin
          Inc(Len, Length(Dependencies[i].Name));
          if Dependencies[i].IsGroup then Inc(Len);
        end;
        Inc(Len, Dependencies.Count * 2);
        if Len <> 0 then
        begin
          SetLength(Result, Len);
          P := @Result[1];
          for i := 0 to Dependencies.Count - 1 do
          begin
            if Dependencies[i].IsGroup then
            begin
              P^ := SC_GROUP_IDENTIFIER;
              Inc(P);
            end;
            P := StrECopy(P, PChar(Dependencies[i].Name));
            Inc(P);
            P^ := #0;
            Inc(P);
          end;
        end;
      end;
    end;

    function GetNTServiceType: Integer;
    const
      NTServiceType: array[TServiceType] of Integer = ( SERVICE_WIN32_OWN_PROCESS,
        SERVICE_KERNEL_DRIVER, SERVICE_FILE_SYSTEM_DRIVER);
    begin
      with Service do begin
        Result := NTServiceType[ServiceType];
        if (ServiceType = stWin32) and Interactive then
          Result := Result or SERVICE_INTERACTIVE_PROCESS;
        if (ServiceType = stWin32) and (Application.ServiceCount > 1) then
          Result := (Result xor SERVICE_WIN32_OWN_PROCESS) or SERVICE_WIN32_SHARE_PROCESS;
      end;
    end;

    function GetNTStartType: Integer;
    const
      NTStartType: array[TStartType] of Integer = (SERVICE_BOOT_START,
        SERVICE_SYSTEM_START, SERVICE_AUTO_START, SERVICE_DEMAND_START,
        SERVICE_DISABLED);
    begin
      with Service do begin
        Result := NTStartType[StartType];
        if (StartType in [stBoot, stSystem]) and (ServiceType <> stDevice) then
          Result := SERVICE_AUTO_START;
      end;
    end;

    function GetNTErrorSeverity: Integer;
    const
      NTErrorSeverity: array[TErrorSeverity] of Integer = (SERVICE_ERROR_IGNORE,
        SERVICE_ERROR_NORMAL, SERVICE_ERROR_SEVERE, SERVICE_ERROR_CRITICAL);
    begin
      with Service do
        Result := NTErrorSeverity[ErrorSeverity];
    end;

  var
    TmpTagID, Svc: Integer;
    PTag, PSSN: Pointer;
    Path: string;
  begin
    Path := ParamStr(0);
    with Service do
    begin
      if Assigned(BeforeInstall) then BeforeInstall(Service);
      TmpTagID := TagID;
      if TmpTagID > 0 then PTag := @TmpTagID else PTag := nil;
      if ServiceStartName = '' then
        PSSN := nil else
        PSSN := PChar(ServiceStartName);
      Svc := CreateService(SvcMgr, PChar(Name), PChar(DisplayName),
        SERVICE_ALL_ACCESS, GetNTServiceType, GetNTStartType, GetNTErrorSeverity,
        PChar(Path), PChar(LoadGroup), PTag, PChar(GetNTDependencies),
        PSSN, PChar(Password));
      TagID := TmpTagID;
      if Svc = 0 then
        RaiseLastWin32Error;
      try
        try
          if Assigned(AfterInstall) then AfterInstall(Service);
        except
          on E: Exception do
          begin
            DeleteService(Svc);
            raise;
          end;
        end;
      finally
        CloseServiceHandle(Svc);
      end;
    end;
  end;

  procedure UninstallService(Service: TService; SvcMgr: Integer);
  var
    Svc: Integer;
  begin
    with Service do
    begin
      if Assigned(BeforeUninstall) then BeforeUninstall(Service);
      Svc := OpenService(SvcMgr, PChar(Name), SERVICE_ALL_ACCESS);
      if Svc = 0 then RaiseLastWin32Error;
      try
        if not DeleteService(Svc) then RaiseLastWin32Error;
      finally
        CloseServiceHandle(Svc);
      end;
      if Assigned(AfterUninstall) then AfterUninstall(Service);
    end;
  end;


var
  SvcMgr: Integer;
  i: Integer;
  Success: Boolean;
  Msg: string;
begin
  with SvcApp do begin
    Success := True;
    SvcMgr := OpenSCManager(nil, nil, SC_MANAGER_ALL_ACCESS);
    if SvcMgr = 0 then RaiseLastWin32Error;
    try
      for i := 0 to ComponentCount - 1 do
        if Components[i] is TService then
        try
          if Install then
            InstallService(TService(Components[i]), SvcMgr) else
            UninstallService(TService(Components[i]), SvcMgr)
        except
          on E: Exception do
          begin
            Success := False;
            if Install then
              Msg := SServiceInstallFailed else
              Msg := SServiceUninstallFailed;
            with TService(Components[i]) do
              if not(Quiet) then
                MessageDlg(Format(Msg, [DisplayName, E.Message]), mtError, [mbOK],0);
          end;
        end;
    finally
      if Success and not(Quiet) then
        if Install then
          MessageDlg(SServiceInstallOK, mtInformation, [mbOk], 0) else
          MessageDlg(SServiceUninstallOK, mtInformation, [mbOk], 0);
      CloseServiceHandle(SvcMgr);
    end;
  end;
end;

procedure InstallServices(SvcApp: TServiceApplication; Quiet: Boolean);
begin
  RegisterServices(SvcApp,true,Quiet);
end;

procedure UninstallServices(SvcApp: TServiceApplication; Quiet: Boolean);
begin
  RegisterServices(SvcApp,false,Quiet);
end;

procedure ServiceApplicationRun(SvcApp: TServiceApplication);
begin
  if FindSwitch('INSTALL') then
    InstallServices(SvcApp,FindSwitch('Q'))
  else
    if FindSwitch('UNINSTALL') then
      UninstallServices(SvcApp,FindSwitch('Q'))
    else
      SvcApp.Run;
end;

end.
