{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agToolButton;

interface

uses Classes, ComCtrls, Messages, Controls;

type
  TagToolButton = class(TToolButton)
  private
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure MouseEnter;
    procedure MouseLeave;
  protected
    FMouseInControl: boolean;
    procedure SetParent(AParent: TWinControl); override;
    procedure Paint; override;
  published
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

implementation

uses Windows, Graphics {$IFDEF D4}, ImgList {$ENDIF};

{ TagToolButton }

procedure TagToolButton.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not FMouseInControl and Enabled then begin
    FMouseInControl := True;
    MouseEnter;
  end;
end;

procedure TagToolButton.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FMouseInControl and Enabled then begin
    FMouseInControl := False;
    MouseLeave;
  end;
end;

procedure TagToolButton.MouseEnter;
begin
  Paint;
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TagToolButton.MouseLeave;
begin
  Paint;
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

var
  Pattern: TBitmap;

procedure CreateBrushPattern;
var
  X, Y: Integer;
begin
  Pattern := TBitmap.Create;
  with Pattern do begin
    Width := 8;
    Height := 8;
    with Canvas do begin
      Brush.Style := bsSolid;
      Brush.Color := GetSysColor(COLOR_BTNFACE);
      FillRect (Rect(0, 0, Width, Height));
      for Y := 0 to 7 do
        for X := 0 to 7 do
          if Odd(Y) = Odd(X) then
            Pixels[X, Y] := clWhite;
    end;
  end;
end;

procedure TagToolButton.SetParent(AParent: TWinControl);
begin
  if Assigned(AParent) and not(AParent is TToolBar) then
    raise EInvalidOperation.Create('A tool button can only be placed on a TToolBar');
  inherited;
end;

procedure TagToolButton.Paint;
var
  R: TRect;
  Offset,TxtOffset: TPoint;
  Bmp: TBitmap;
  {$IFDEF D4}
  ImgList: TCustomImageList;
  {$ELSE}
  ImgList: TImageList;
  {$ENDIF}
begin
  if FToolBar.Flat and (Style = tbsCheck) and Enabled and Down and
    not(csDesigning in ComponentState) then
  begin
    Bmp:=TBitmap.Create;
    try
      Bmp.Width:=Width;
      Bmp.Height:=Height;
      Bmp.Canvas.Font.Assign(Font);
      with Bmp.Canvas do begin
        R:=Rect(0, 0, Width, Height);
        Brush.Color:=Color;
        FillRect(R);
        DrawEdge(Handle, R, BDR_SUNKENOUTER, BF_RECT);
        InflateRect(R, -1, -1);
        if not FMouseInControl then begin
          if Pattern = nil then CreateBrushPattern;
          Brush.Bitmap:=Pattern;
          FillRect(R);
        end;
        if FToolBar.List then
          TxtOffset.x:=8
        else
          TxtOffset.x:=(Width-TextWidth(Caption)) div 2 + 1;
        TxtOffset.y:=5;

        if Assigned(FToolBar.HotImages) then
          ImgList:=FToolBar.HotImages
        else
          if Assigned(FToolBar.Images) then
            ImgList:=FToolBar.Images
          else
            ImgList:=nil;

        if Assigned(ImgList) then begin
          if FToolBar.List then begin
            Offset.x:=6;
            inc(TxtOffset.x,ImgList.Width);
          end else begin
            Offset.x:=(Width-ImgList.Width) div 2 + 1;
            inc(TxtOffset.y,ImgList.Height);
          end;
          Offset.y:=4;
          if ImageIndex >= 0 then
            ImgList.Draw(Bmp.Canvas,Offset.x,Offset.y,
              ImageIndex{$IFDEF D4},true{$ENDIF});
        end;
        if FToolBar.ShowCaptions then begin
          Brush.Style:=bsClear;
          TextOut(TxtOffset.x,TxtOffset.y,Caption);
        end;
      end;
      Canvas.Copyrect(rect(0,0,Width,Height),Bmp.Canvas,rect(0,0,Width,Height));
    finally
      Bmp.Free;
    end;
  end else
    inherited;
end;

initialization
  Pattern:=nil;
  
finalization
  if Assigned(Pattern) then Pattern.Free;

end.

