{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agURLLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ShellAPI;

type
  TagURLLabel = class(TCustomLabel)
  private
    { Private declarations }
    FURL: string;
    FFlat: boolean;
    FActColor,FPasColor: TColor;
    FMouseInControl: boolean;
    procedure FlatChanged(Value: Boolean);
    procedure ActColorChanged(Value: TColor);
    procedure PasColorChanged(Value: TColor);
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property Alignment;
    property AutoSize;
    property Caption;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property URL: string read FURL write FURL;
    property Flat: boolean read FFlat write FlatChanged;
    property ActiveColor: TColor read FActColor write ActColorChanged;
    property PassiveColor: TColor read FPasColor write PasColorChanged;
  end;

implementation

constructor TagURLLabel.Create(AOwner: TComponent);
begin
  inherited;
  Caption:='aghost@bigfoot.com';
  FURL:='aghost@bigfoot.com';
  FMouseInControl:=false;
  Cursor:=crHandPoint;
  Font.Style:=[fsUnderline];
  FFlat:=true;
  FActColor:=clBlue; FPasColor:=clNavy;
end;

procedure TagURLLabel.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  s: string;
begin
  inherited;
  if (Button = mbLeft) and Enabled then
   begin
     if Pos('@',FURL) = 0 then
       s:=Format('http://%s',[FURL])
     else
       s:=Format('mailto:%s',[FURL]);
     ShellExecute(0,nil,PChar(s),nil,nil,SW_SHOWMAXIMIZED);
   end;
end;

procedure TagURLLabel.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  P: TPoint;
begin
  inherited;
  P := ClientToScreen(Point(X, Y));
  if (FindDragTarget(P,false)=Self) and not(FMouseInControl) then
   begin
     FMouseInControl:=true;
     Repaint;
   end;
end;

procedure TagURLLabel.CMMouseLeave(var Message: TMessage);
begin
  if FMouseInControl then
   begin
     FMouseInControl:=false;
     Repaint;
   end;
end;

procedure TagURLLabel.Paint;
begin
  if FFlat then
   begin
     if FMouseInControl then
       Font.Color:=FActColor
     else
       Font.Color:=FPasColor;
   end
  else
    Font.Color:=FPasColor;
  inherited;
end;

procedure TagURLLabel.FlatChanged(Value: Boolean);
begin
  if FFlat <> Value then
   begin
     FFlat:=Value;
     Repaint;
   end;
end;

procedure TagURLLabel.ActColorChanged(Value: TColor);
begin
  if FActColor <> Value then
   begin
     FActColor:=Value;
     Repaint;
   end;
end;

procedure TagURLLabel.PasColorChanged(Value: TColor);
begin
  if FPasColor <> Value then
   begin
     FPasColor:=Value;
     Repaint;
   end;
end;

end.


