{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agUtils2;

interface

uses Windows, agUtils;

// ExitWindowsEx wrappers
function LogOffWindows(Force: boolean): boolean;
function PowerOffComputer(Force: boolean): boolean;
function RebootComputer(Force: boolean): boolean;
function ShutdownComputer(Force: boolean): boolean;
function DownWindows(Flags: UINT): boolean;

// file version compare functions
function GetFileVersion(const FileName: string; var HighVer,LowVer: dword): boolean;
function IsTargetNewer(const FileSource,FileTarget: string): boolean;
function IsTargetNewer2(FileSourceDate: TDateTime; const FileSourceVer: string;
  const FileTarget: string): boolean;

// miscellaneous functions
function HTMLHelpInstalled: Boolean;
function RegisterServer(const ModuleName: string): boolean;
function UnregisterServer(const ModuleName: string): boolean;

implementation

uses SysUtils, agArithm, agStrUtils;

function DownWindows(Flags: UINT): boolean;
var
  TokenPriv: TTokenPrivileges;
  TokenHandle: THandle;
begin
  Result:=false;
  if (Win32Platform = VER_PLATFORM_WIN32_NT) and
    ((Flags and (EWX_POWEROFF or EWX_REBOOT or EWX_SHUTDOWN)) <> 0) then
  begin
    if OpenProcessToken(GetCurrentProcess,TOKEN_ADJUST_PRIVILEGES,TokenHandle) then
      if LookupPrivilegeValue(nil,'SeShutdownPrivilege',TokenPriv.Privileges[0].LUID) then begin
        TokenPriv.PrivilegeCount:=1;
        TokenPriv.Privileges[0].Attributes:=SE_PRIVILEGE_ENABLED;
        if not AdjustTokenPrivileges(TokenHandle,false,TokenPriv,0,
          TTokenPrivileges(nil^),DWORD(nil^)) then exit;
      end;
  end;
  Result:=ExitWindowsEx(Flags,0);
end;

const
  ForceFlag: array [boolean] of UINT = (0, EWX_FORCE);

function LogOffWindows(Force: boolean): boolean;
begin
  Result:=DownWindows(EWX_LOGOFF or ForceFlag[Force]);
end;

function PowerOffComputer(Force: boolean): boolean;
begin
  Result:=DownWindows(EWX_POWEROFF or ForceFlag[Force]);
end;

function RebootComputer(Force: boolean): boolean;
begin
  Result:=DownWindows(EWX_REBOOT or ForceFlag[Force]);
end;

function ShutdownComputer(Force: boolean): boolean;
begin
  Result:=DownWindows(EWX_SHUTDOWN or ForceFlag[Force]);
end;

// miscellaneous functions

function HTMLHelpInstalled: boolean;
var
  HHCTRL: THandle;
begin
  HHCTRL:=LoadLibrary('HHCTRL.OCX');
  if HHCTRL <> 0 then begin
    FreeLibrary(HHCTRL);
    Result:=true;
  end else
    Result:=false;
end;

function RegisterServer(const ModuleName: string): boolean;
type
  TProc = procedure;
var
  Handle: THandle;
  RegProc: Pointer;
begin
  Result:=false;
  Handle:=LoadLibrary(PChar(ModuleName));
  if Handle = 0 then exit;
  try
    RegProc:=GetProcAddress(Handle,'DllRegisterServer');
    if Assigned(RegProc) then begin
      TProc(RegProc);
      Result:=true;
    end;
  finally
    FreeLibrary(Handle);
  end;
end;

function UnregisterServer(const ModuleName: string): boolean;
type
  TProc = procedure;
var
  Handle: THandle;
  RegProc: Pointer;
begin
  Result:=false;
  Handle:=LoadLibrary(PChar(ModuleName));
  if Handle = 0 then exit;
  try
    RegProc:=GetProcAddress(Handle,'DllUnregisterServer');
    if Assigned(RegProc) then begin
      TProc(RegProc);
      Result:=true;
    end;
  finally
    FreeLibrary(Handle);
  end;
end;

function GetFileVersion(const FileName: string; var HighVer,LowVer: dword): boolean;
var
  size,data: DWORD;
  buffer: pointer;
  FileInfo: PVSFixedFileInfo;
begin
  Result:=false;
  size:=GetFileVersionInfoSize(PChar(FileName),data);
  if size = 0 then exit;
  GetMem(buffer,size+1);
  try
    GetFileVersionInfo(PChar(FileName),0,size,buffer);
    if not VerQueryValue(buffer,'\',pointer(FileInfo),data) then exit;
    HighVer:=FileInfo^.dwFileVersionMS;
    LowVer:=FileInfo^.dwFileVersionLS;
    Result:=true;
  finally
    FreeMem(buffer);
  end;
end;

function IsTargetNewer(const FileSource,FileTarget: string): boolean;
var
  hs,ls,ht,lt: dword;
begin
  Result:=false;
  if not FileExists(FileTarget) then exit;
  if GetFileVersion(FileSource,hs,ls) and GetFileVersion(FileTarget,ht,lt) then begin
    if (ht > hs) or ((ht = hs) and (lt > ls)) then Result:=true;
  end else
    if FileDateToDateTime(FileAge(FileTarget)) > FileDateToDateTime(FileAge(FileSource))
      then Result:=true;
end;

function IsTargetNewer2(FileSourceDate: TDateTime; const FileSourceVer: string;
  const FileTarget: string): boolean;
var
  hs,ls,ht,lt: dword;
  CheckDate: boolean;
begin
  Result:=false;
  if not FileExists(FileTarget) then exit;
  CheckDate:=true;
  if FileSourceVer <> '' then begin
    hs:=(StrToIntDef(GetWord(1,FileSourceVer,['.']),0) shl 16)+
      StrToIntDef(GetWord(2,FileSourceVer,['.']),0);
    ls:=(StrToIntDef(GetWord(3,FileSourceVer,['.']),0) shl 16)+
      StrToIntDef(GetWord(4,FileSourceVer,['.']),0);
    if GetFileVersion(FileTarget,ht,lt) then begin
      if (ht > hs) or ((ht = hs) and (lt > ls)) then Result:=true;
      CheckDate:=false;
    end;
  end;
  if CheckDate then
    if FileDateToDateTime(FileAge(FileTarget)) > FileSourceDate then
      Result:=true;
end;

end.
