{*******************************************************}
{                                                       }
{       Alex Ghost Library                              }
{                                                       }
{       Copyright (c) 1999,2000 Alexey Popov            }
{                                                       }
{*******************************************************}

{$I AG.INC}

unit agVCLUtils;

interface

uses SysUtils, Windows, Messages, Classes, Graphics, Forms, Controls, ComCtrls;

// ListView functions
function ListViewSort(Item1, Item2: TListItem; Column, Direction: integer): integer;
procedure AdjustListViewWidth(Lst: TListView);

// System image list routines
procedure GetLargeSysImageList(L: TImageList);
procedure GetSmallSysImageList(L: TImageList);
function GetSysImageIndex(Filename: String): Integer;

// make icon function
function MakeIcon(ResID: string): TIcon;
function MakeIconID(ResID: Word): TIcon;
function MakeIconSize(ResID: string; CX,CY: integer): TIcon;
function MakeModuleIcon(Module: THandle; ResID: string): TIcon;
function MakeModuleIconSize(Module: THandle; ResID: string; CX,CY: integer): TIcon;
function MakeSmallIcon(ResID: string): TIcon;
function MakeLargeIcon(ResID: string): TIcon;


// miscellaneous functions

const
  FM_INITFORM = WM_USER + 141;
  FM_DONEFORM = WM_USER + 142;

type
  TInitDoneMsg = packed record
    Msg: Cardinal;
    Params: PVariant;
    LParam: Longint;
    Result: Longint;
  end;

function ShowDialog(FormClass: TFormClass): boolean;
function ShowDialogParam(FormClass: TFormClass; var Params: Variant): boolean;

procedure FreeForm(var Form: TForm);
procedure SetFocusControl(Control: TWinControl);
procedure PopupDefaultClick(Control: TControl);
procedure Delay(MSecs: Longint);

// Standard Windows colors that are not defined by Delphi
const
  clCream = TColor($A6CAF0);
  clMoneyGreen = TColor($C0DCC0);
  clSkyBlue = TColor($FFFBF0);

// Cursor routines
const
  WaitCursor: TCursor = crHourGlass;

procedure StartWait;
procedure StopWait;

// font <--> string conversion
function FontToStr(Font: TFont): string;
procedure StrToFont(const Value: string; Font: TFont);


// ini/reg functions (in addition to agUtils)

procedure IniReadFont(const IniFile,Section,Key: string; Value: TFont);
function IniWriteFont(const IniFile,Section,Key: string; Value: TFont): boolean;

procedure RegGetFont(const Path: string; Value: TFont);
function RegSetFont(const Path: string; Value: TFont): boolean;


implementation

uses Menus, CommCtrl, ShellApi, {$IFDEF D4} ActnList, {$ENDIF}
  agArithm, agDateUtils, agStrUtils, agUtils, TypInfo, agRegIni;

type
  THack = class(TCustomControl);

procedure Delay(MSecs: Longint);
var
  FirstTickCount, Now: Longint;
begin
  FirstTickCount := GetTickCount;
  repeat
    Application.ProcessMessages;
    Now := GetTickCount;
  until (Now - FirstTickCount >= MSecs) or (Now < FirstTickCount);
end;

const
  WaitCount: Integer = 0;
  SaveCursor: TCursor = crDefault;

procedure StartWait;
begin
  if WaitCount = 0 then begin
    SaveCursor := Screen.Cursor;
    Screen.Cursor := WaitCursor;
  end;
  Inc(WaitCount);
end;

procedure StopWait;
begin
  if WaitCount > 0 then begin
    Dec(WaitCount);
    if WaitCount = 0 then Screen.Cursor := SaveCursor;
  end;
end;

procedure FreeForm(var Form: TForm);
begin
  if Assigned(Form) then begin
    Form.Free;
    Form:=nil;
  end;
end;

procedure SetFocusControl(Control: TWinControl);
var
  MayEnable: boolean;
  Par: TWinControl;
begin
  MayEnable:=true;
  Par:=Control;
  while Assigned(Par.Parent) do begin
    Par:=Par.Parent;
    if Par is TForm then begin
      if (GetForegroundWindow <> Par.Handle) or
        ((Par as TForm).WindowState = wsMinimized) then
        MayEnable:=false;
      break;
    end;
  end;
  try
    if MayEnable then Control.SetFocus;
  except
  end;
end;

procedure PopupDefaultClick(Control: TControl);
var
  i: integer;
begin
  if THack(Control).PopupMenu = nil then exit;
  with THack(Control).PopupMenu.Items do
    for i:=0 to Count-1 do
      if Items[i].Default then Items[i].Click;
end;

function ListViewSort(Item1, Item2: TListItem; Column, Direction: integer): integer;

  function IsValidNumber(const S: string; var V: extended): boolean;
  var
    NumCode: integer;
  begin
    Val(S, V, NumCode);
    Result:=(NumCode = 0);
  end;

  function IsValidDate(const S: string; var D: TDateTime): boolean;
  begin
    D:=StrToDateTimeDef(S,0);
    if D = 0 then Result:=false else Result:=true;
  end;

var
  Str1,Str2: string;
  Val1,Val2: extended;
  Date1,Date2: TDateTime;
begin
  if Column = 0 then begin
    Str1:=Item1.Caption;
    Str2:=Item2.Caption;
  end else begin
    if Item1.SubItems.Count > Column-1 then Str1:=Item1.SubItems[Column-1] else Str1:='';
    if Item2.SubItems.Count > Column-1 then Str2:=Item2.SubItems[Column-1] else Str2:='';
  end;
  if IsValidNumber(Str1,Val1) and IsValidNumber(Str2,Val2) then
    if Val1 < Val2 then
      Result:=-1
    else
      if Val1 > Val2 then
        Result:=1
      else
        Result:=0
  else
    if IsValidDate(Str1,Date1) and IsValidDate(Str2,Date2) then
      Result:=Sgn(Date1-Date2)
    else begin
      Result:=AnsiCompareStr(Str1,Str2);
      if (Str1<>'') and (Str2='') then Result:=-1;
      if (Str1='') and (Str2<>'') then Result:=1;
    end;
  Result:=Direction*Result;
end;

procedure AdjustListViewWidth(Lst: TListView);
var
  i,w1,w2: integer;
begin
  with Lst do begin
    if ViewStyle <> vsReport then Exit;
    Items.BeginUpdate;
    try
      for i:=0 to Columns.Count-1 do begin
        ListView_SetColumnWidth(Lst.Handle,i,-1);
        w1:=ListView_GetColumnWidth(Lst.Handle,i);
        Columns[i].Width:=w1;
        w2:=Lst.StringWidth(Columns[i].Caption)+20;
        Columns[i].Width:=iif(w1>w2,w1,w2);
      end;
    finally
      Items.EndUpdate;
    end;
  end;
end;

function ShowDialog(FormClass: TFormClass): boolean;
var
  Form: TForm;
begin
  Application.CreateForm(FormClass,Form);
  try
    Result:=(Form.ShowModal in [mrOk,mrYes]);
  finally
    FreeForm(Form);
  end;
end;

function ShowDialogParam(FormClass: TFormClass; var Params: Variant): boolean;
var
  Form: TForm;
begin
  Application.CreateForm(FormClass,Form);
  try
    SendMessage(Form.Handle,FM_INITFORM,integer(@Params),0);
    Result:=(Form.ShowModal in [mrOk,mrYes]);
    if Result then SendMessage(Form.Handle,FM_DONEFORM,integer(@Params),0);
  finally
    FreeForm(Form);
  end;
end;

// System image list routines
procedure GetLargeSysImageList(L: TImageList);
var
  AHandle: DWORD;
  Fileinfo: TSHFileInfo;
begin
  AHandle := SHGetFileInfo('', 0, Fileinfo, sizeof(TSHFileInfo),
    SHGFI_LARGEICON or SHGFI_SYSICONINDEX);
  if AHandle <> 0 then begin
    L.Handle := AHandle;
    L.ShareImages := True;
  end;
end;

procedure GetSmallSysImageList(L: TImageList);
var
  AHandle: DWORD;
  Fileinfo: TSHFileInfo;
begin
  AHandle := SHGetFileInfo('', 0, Fileinfo, sizeof(TSHFileInfo),
    SHGFI_SMALLICON or SHGFI_SYSICONINDEX);
  if AHandle <> 0 then begin
    L.Handle := AHandle;
    L.ShareImages := True;
  end;
end;

function GetSysImageIndex(Filename: String): Integer;
var
  FileInfo: TSHFileInfo;
begin
  if SHGetFileInfo(PChar(FileName), 0, FileInfo, sizeof(TSHFileInfo),
    SHGFI_ICON or SHGFI_SYSICONINDEX) <> 0
  then Result := FileInfo.IIcon
  else Result := 0;
end;

// make icon functions

function MakeIcon(ResID: string): TIcon;
begin
  Result := MakeModuleIcon(hInstance, ResID);
end;

function MakeIconID(ResID: Word): TIcon;
begin
  Result := MakeModuleIcon(hInstance, MakeIntResource(ResID));
end;

function MakeModuleIcon(Module: THandle; ResID: string): TIcon;
begin
  Result := TIcon.Create;
  Result.Handle := LoadIcon(Module, PChar(ResID));
  if Result.Handle = 0 then begin
    Result.Free;
    Result := nil;
  end;
end;

function MakeIconSize(ResID: string; CX,CY: integer): TIcon;
begin
  Result:=MakeModuleIconSize(hInstance,ResID,CX,CY);
end;

function MakeModuleIconSize(Module: THandle; ResID: string; CX,CY: integer): TIcon;
begin
  Result:=TIcon.Create;
  Result.Handle:=LoadImage(Module,PChar(ResID),IMAGE_ICON,CX,CY,LR_DEFAULTCOLOR);
  if Result.Handle = 0 then begin
    Result.Free;
    Result := nil;
  end;
end;

function MakeSmallIcon(ResID: string): TIcon;
begin
  Result:=MakeIconSize(ResID,16,16);
end;

function MakeLargeIcon(ResID: string): TIcon;
begin
  Result:=MakeIconSize(ResID,32,32);
end;


// FontStyleToStr, StrToFontStyle, FontToStr, StrToFont is taken from SohoLib

function FontStyleToStr(Value: TFontStyles): string;
begin
  Result := '';
  if Value = [] then Result := 'N';
  if fsBold in Value then Result := Result + 'B';
  if fsItalic in Value then Result := Result + 'I';
  if fsUnderline in Value then Result := Result + 'U';
  if fsStrikeOut in Value then Result := Result + 'S';
end;

function StrToFontStyle(const Value: string): TFontStyles;
var
  i: integer;
begin
  Result := [];
  for i := 1 to Length(Value) do
    case Value[i] of
      'B': Result := Result + [fsBold];
      'I': Result := Result + [fsItalic];
      'U': Result := Result + [fsUnderline];
      'S': Result := Result + [fsStrikeOut];
    end;
end;

function FontToStr(Font: TFont): string;
begin
  Result := Font.Name + ',' + IntToStr(Font.Size) + ',' +
    ColorToString(Font.Color) + ',' + FontStyleToStr(Font.Style);
end;

procedure StrToFont(const Value: string; Font: TFont);
begin
  if Font = nil then exit;
  with Font do begin
    Name := GetWord(1,Value,[',']);
    Size := StrToIntDef(GetWord(2,Value,[',']),8);
    Color := StrToColor(GetWord(3,Value,[',']));
    Style := StrToFontStyle(GetWord(4,Value,[',']));
  end;
end;

procedure IniReadFont(const IniFile,Section,Key: string; Value: TFont);
begin
  StrToFont(IniReadStr(IniFile,Section,Key,FontToStr(Value)),Value);
end;

function IniWriteFont(const IniFile,Section,Key: string; Value: TFont): boolean;
begin
  Result:=IniWriteStr(IniFile,Section,Key,FontToStr(Value));
end;

procedure RegGetFont(const Path: string; Value: TFont);
begin
  StrToFont(RegGetStr(Path,FontToStr(Value)),Value);
end;

function RegSetFont(const Path: string; Value: TFont): boolean;
begin
  Result:=RegSetStr(Path,FontToStr(Value));
end;

end.
